/* /////////////////////////////////////////////////////////////////////////
 * File:        example_c_1.c
 *
 * Purpose:     C example program for the recls core library. Demonstrates:
 *
 *                - searching (via Recls_Search()) for files
 *                - recursive operation
 *                - display of full path of each entry
 *                - handling of errors and reporting of error information
 *                - elicitation of entry properties via API function calls
 *
 * Created:     29th May 2006
 * Updated:     18th June 2006
 *
 * www:         http://www.recls.org/
 *
 * License:     Copyright (c) 2006, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              (Licensed under the Synesis Software Open License)
 *
 *              This source code is placed into the public domain 2006
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 * ////////////////////////////////////////////////////////////////////// */

/* recls Header Files */
#include <recls/recls.h>

/* Standard C Library Files */
#include <stdio.h>      /* for printf() / fprintf()         */
#include <stdlib.h>     /* for EXIT_SUCCESS / EXIT_FAILURE  */
#include <string.h>

/* /////////////////////////////////////////////////////////////////////////
 * Macros and definitions
 */

#ifndef RECLS_NUM_ELEMENTS
# define RECLS_NUM_ELEMENTS(x)          (sizeof(x) / sizeof((x)[0]))
#endif /* !RECLS_NUM_ELEMENTS */

/* ////////////////////////////////////////////////////////////////////// */

int main()
{
    /* Declare a search handle, define the flags (for recursive file search)
     * and start a search.
     */
    hrecls_t        hSrch;
    recls_uint32_t  flags   =   RECLS_F_FILES | RECLS_F_RECURSIVE;
    recls_rc_t      rc      =   Recls_Search(".", Recls_GetWildcardsAll(), flags, &hSrch);

    if(RECLS_RC_NO_MORE_DATA == rc)
    {
        printf("  no matches found\n");

        return EXIT_SUCCESS;
    }
    else if(RECLS_FAILED(rc))
    {
        /* The search failed. Display the error string. */
        char    err[1001];
        size_t  n   =   Recls_GetErrorString(rc, &err[0], sizeof(err) - 1);

        err[n] = '\0';

        fprintf(stderr, "Search failed: %s\n", err);

        return EXIT_FAILURE;
    }
    else
    {
        /* Get the details for the first entry, ... */ 

        recls_info_t    entry;

        Recls_GetDetails(hSrch, &entry);

        do
        {
            /* ... display the full path, ... */

            char    path[1001];
            size_t  cch;

            cch = Recls_GetPathProperty(entry, &path[0], RECLS_NUM_ELEMENTS(path));
            printf("%.*s\n", (int)cch, &path[0]);

            /* ... close the entry handle, ... */
            Recls_CloseDetails(entry);

        } /* ... and get the next entry. */
        while(RECLS_SUCCEEDED(Recls_GetNextDetails(hSrch, &entry)));

        /* Close the search handle. */
        Recls_SearchClose(hSrch);

        return EXIT_SUCCESS;
    }
}

/* ////////////////////////////////////////////////////////////////////// */
