/* /////////////////////////////////////////////////////////////////////////////
 * File:        example_c_8.c
 *
 * Purpose:     C example program for the recls core library. Demonstrates:
 *
 *                - elicitation of all roots (via Recls_GetRoots()), and
 *                  selected roots (via Recls_GetSelectedRoots()), on host
 *                  system
 *                - determination of total size of all files per root (via
 *                  Recls_CalcDirectorySize())
 *
 * Created:     17th June 2006
 * Updated:     18th June 2006
 *
 * www:         http://www.recls.org/
 *
 * License:     Copyright (c) 2006, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              (Licensed under the Synesis Software Open License)
 *
 *              This source code is placed into the public domain 2006
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 * ////////////////////////////////////////////////////////////////////// */

/* recls Header Files */
#include <recls/recls.h>

/* Standard C Library Files */
#include <stdio.h>      /* for printf() / fprintf()         */
#include <stdlib.h>     /* for EXIT_SUCCESS / EXIT_FAILURE  */
#include <string.h>

/* /////////////////////////////////////////////////////////////////////////
 * Macros and definitions
 */

#ifndef RECLS_NUM_ELEMENTS
# define RECLS_NUM_ELEMENTS(x)          (sizeof(x) / sizeof((x)[0]))
#endif /* !RECLS_NUM_ELEMENTS */

/* ////////////////////////////////////////////////////////////////////// */

int main()
{
    /* Display all the roots on the host system. */
    {
        recls_root_t    roots[26];
        size_t          numRoots    =   Recls_GetRoots(&roots[0], RECLS_NUM_ELEMENTS(roots));
        size_t          i;

        printf("All roots on host system:\n");
        for(i = 0; i < numRoots; ++i)
        {
            printf("  %s\n", roots[i].name);
        }
        printf("\n");
    }

    /* Display the fixed, optical and ram-drive roots on the host system,
     * and calculate their size.
     */
    {
        unsigned        rootTypes   =   RECLS_ROOTS_F_FIXED_DRIVES | RECLS_ROOTS_F_CDROM_DRIVES | RECLS_ROOTS_F_RAM_DRIVES;
        size_t          numRoots    =   Recls_GetSelectedRoots(NULL, 0, rootTypes);
        recls_root_t    *roots      =   (recls_root_t*)malloc(sizeof(recls_root_t) * numRoots);
        size_t          i;

        numRoots = Recls_GetSelectedRoots(&roots[0], numRoots, rootTypes);

        printf("Measuring root directories of selected roots on host system:\n");
        for(i = 0; i < numRoots; ++i)
        {
#if defined(RECLS_PLATFORM_IS_WIN32)
            unsigned long   size = (unsigned long)Recls_CalcDirectorySize(roots[i].name).QuadPart;
#else /* ? OS */
            unsigned long   size = (unsigned long)Recls_CalcDirectorySize(roots[i].name);
#endif /* OS */

            printf("  %s : %lu\n", roots[i].name, (unsigned long)size);
        }
        printf("\n");

        free(roots);
    }

    return EXIT_SUCCESS;
}

/* ////////////////////////////////////////////////////////////////////// */
