/* /////////////////////////////////////////////////////////////////////////////
 * File:        example_stl_1.cpp
 *
 * Purpose:     STL example program for recls/STL. Demonstrates:
 *
 *                - searching for files
 *                - recursive operation
 *                - display of full path of each entry
 *                - handling exceptions and reporting of error information
 *                - elicitation of entry properties via method calls
 *
 * Created:     18th June 2006
 * Updated:     18th June 2006
 *
 * www:         http://www.recls.org/
 *
 * License:     Copyright (c) 2006, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              (Licensed under the Synesis Software Open License)
 *
 *              This source code is placed into the public domain 2006
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software.
 *
 * ////////////////////////////////////////////////////////////////////// */


/* recls Header Files */
#include <recls/stl/search_sequence.hpp>
#include <recls/implicit_link.h>

/* Standard C++ Header Files */
#include <exception>
#include <iostream>
#include <iterator>

/* Standard C Header Files */
#include <stdlib.h>     /* for EXIT_SUCCESS / EXIT_FAILURE  */

/* ////////////////////////////////////////////////////////////////////////// */

int main()
{
    try
    {
        recls::uint32_t             flags   =   recls::RECLS_F_FILES | recls::RECLS_F_RECURSIVE;
        recls::stl::search_sequence srch(".", recls::Recls_GetWildcardsAll(), flags);

        std::copy(  srch.begin(), srch.end()
                ,   std::ostream_iterator<recls::stl::search_sequence::value_type>(std::cout, "\n"));
    }
    catch(recls::stl::recls_exception &x)
    {
        std::cerr << "Recls error: " << x.rc() << ", " << x.what() << std::endl;

        return EXIT_FAILURE;
    }
    catch(std::bad_alloc &)
    {
        std::cerr << "Out of memory" << std::endl;

        return EXIT_FAILURE;
    }
    catch(std::exception &x)
    {
        std::cerr << "Unhandled error: " << x.what() << std::endl;

        return EXIT_FAILURE;
    }
    catch(...)
    {
        std::cerr << "Unhandled unknown error" << std::endl;

        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}

/* ////////////////////////////////////////////////////////////////////////// */
