/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls/recls.h
 *
 * Purpose:     Main header file for the recls API.
 *
 * Created:     15th August 2003
 * Updated:     2nd June 2007
 *
 * Home:        http://recls.org/
 *
 * Copyright (c) 2003-2007, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/** \file recls/recls.h
 *
 * \brief [C, C++] The root header for the \ref group__recls API
 */

#ifndef RECLS_INCL_RECLS_H_RECLS
#define RECLS_INCL_RECLS_H_RECLS

/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_RECLS_H_RECLS_MAJOR      3
# define RECLS_VER_RECLS_H_RECLS_MINOR      8
# define RECLS_VER_RECLS_H_RECLS_REVISION   7
# define RECLS_VER_RECLS_H_RECLS_EDIT       81
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \name recls API Version
 * \ingroup group__recls
 */
/** @{ */
/** \def RECLS_VER_MAJOR
 * \brief The major version number of RECLS.
 */

/** \def RECLS_VER_MINOR
 * \brief The minor version number of RECLS.
 */

/** \def RECLS_VER_REVISION
 * \brief The revision version number of RECLS.
 */

/** \def RECLS_VER
 * \brief The current composite version number of RECLS.
 */
/** @} */

/* recls version */
#define RECLS_VER_MAJOR     1
#define RECLS_VER_MINOR     8
#define RECLS_VER_REVISION  10
#define RECLS_VER_1_0_1     0x01000100
#define RECLS_VER_1_1_1     0x01010100
#define RECLS_VER_1_2_1     0x01020100
#define RECLS_VER_1_3_1     0x01030100
#define RECLS_VER_1_4_1     0x01040100
#define RECLS_VER_1_4_2     0x01040200
#define RECLS_VER_1_5_1     0x01050100
#define RECLS_VER_1_5_2     0x01050200
#define RECLS_VER_1_5_3     0x01050300
#define RECLS_VER_1_6_1     0x01060100
#define RECLS_VER_1_6_2     0x01060200
#define RECLS_VER_1_6_3     0x01060300
#define RECLS_VER_1_6_4     0x01060400
#define RECLS_VER_1_6_5     0x01060500
#define RECLS_VER_1_7_1     0x01070100
#define RECLS_VER_1_7_2     0x01070200
#define RECLS_VER_1_8_1     0x01080100
#define RECLS_VER_1_8_2     0x01080200
#define RECLS_VER_1_8_3     0x01080300
#define RECLS_VER_1_8_4     0x01080400
#define RECLS_VER_1_8_5     0x01080500
#define RECLS_VER_1_8_6     0x01080600
#define RECLS_VER_1_8_7     0x01080700
#define RECLS_VER_1_8_8     0x01080800
#define RECLS_VER_1_8_9     0x01080900
#define RECLS_VER_1_8_10    0x01080a00
#define RECLS_VER           RECLS_VER_1_8_10

/* /////////////////////////////////////////////////////////////////////////////
 * Strictness
 */

#ifndef RECLS_NO_STRICT
# define RECLS_STRICT
#endif /* !RECLS_NO_STRICT */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

/* Detects C & C++ things, such as namespace support */
#include <recls/internal/language.h>
/* Includes platform-specific headers */
#include <recls/internal/platform.h>
/* Includes stddef.h / cstddef, and defines the recls types: recls_s/uint8/16/32/64_t */
#include <recls/internal/compiler.h>
/* Defines recls_filesize_t, recls_time_t */
#include <recls/internal/platform_types.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

#if !defined(RECLS_NO_NAMESPACE)
#  define RECLS_QUAL(x)                 ::recls::x
#else
# define RECLS_QUAL(x)                  x
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants and definitions
 */

/** \def RECLS_ROOT_NAME_LEN
 *
 * \brief Defines the number of characters in a system root.
 *
 * \ingroup group__recls
 */
#if defined(RECLS_DOCUMENTATION_SKIP_SECTION)
# define RECLS_ROOT_NAME_LEN            (1) or (3)
#elif defined(RECLS_PLATFORM_IS_WIN32)
# define RECLS_ROOT_NAME_LEN            (3)
#elif defined(RECLS_PLATFORM_IS_UNIX)
# define RECLS_ROOT_NAME_LEN            (1)
#elif defined(RECLS_OVERRIDE_PLATFORM)
  /* Assume that the appropriate inclusions are made */
#else
# error Platform not recognised
#endif /* platform */

/* /////////////////////////////////////////////////////////////////////////////
 * Function specifications
 */

/** \brief Defines the recls linkage and calling convention.
 *
 * \ingroup group__recls
 */
#define RECLS_FNDECL(rt)    RECLS_LINKAGE_C rt RECLS_CALLCONV_DEFAULT

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

/* recls_rc_t */
#ifdef __cplusplus

/** \brief The type of return codes issued by the API functions.
 *
 * \ingroup group__recls
 */
typedef recls_sint32_t                      recls_rc_t;

/** \brief General success code.
 *
 * \ingroup group__recls
 */
const recls_rc_t                            RECLS_RC_OK(0);
/** \brief General failure code.
 *
 * \ingroup group__recls
 */
const recls_rc_t                            RECLS_RC_FAIL(-1);

/** \brief Returns non-zero if the given return code indicates failure.
 *
 * \ingroup group__recls
 */
inline bool RECLS_FAILED(recls_rc_t const &rc)
{
    return rc < 0;
}

/** \brief Returns non-zero if the given return code indicates success.
 *
 * \ingroup group__recls
 */
inline bool RECLS_SUCCEEDED(recls_rc_t const &rc)
{
    return !RECLS_FAILED(rc);
}

#else /* ? __cplusplus */

/** \brief The type of return codes issued by the API functions.
 *
 * \ingroup group__recls
 */
typedef recls_sint32_t                      recls_rc_t;

/** \brief General success code.
 *
 * \ingroup group__recls
 */
# define RECLS_RC_OK                        (0)
/** \brief General failure code.
 *
 * \ingroup group__recls
 */
# define RECLS_RC_FAIL                      (-1)

/** \brief Evaluates to non-zero if the given return code indicates failure.
 *
 * \ingroup group__recls
 */
# define RECLS_FAILED(rc)                   ((rc) < 0)
/** \brief Evaluates to non-zero if the given return code indicates success.
 *
 * \ingroup group__recls
 */
# define RECLS_SUCCEEDED(rc)                (!RECLS_FAILED(rc))

#endif /* __cplusplus */

/* hrecls_t */
#if !defined(RECLS_DOCUMENTATION_SKIP_SECTION) && \
    defined(RECLS_COMPILER_IS_BORLAND) && \
    __BORLANDC__ < 0x0560
struct hrecls_t_
{
    int dummy;
};
#else /* ? compiler */
struct hrecls_t_;
#endif /* compiler */

/** \brief The handle to a recursive search operation.
 *
 * \ingroup group__recls
 */
typedef struct hrecls_t_ const *            hrecls_t;

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

/* Defines result codes */
#include <recls/internal/retcodes.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
namespace recls
{
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Enumerations
 */

/** \brief Search flags
 *
 * \ingroup group__recls
 *
 * These flags moderate the search behaviour of the
 * Recls_Search(), Recls_SearchFeedback(), and Recls_SearchProcess() functions.
 */
enum RECLS_FLAG
{
        RECLS_F_FILES                       =   0x00000001  /*!< Include files in search. Included by default if none specified */
    ,   RECLS_F_DIRECTORIES                 =   0x00000002  /*!< Include directories in search */
    ,   RECLS_F_LINKS                       =   0x00000004  /*!< Include links in search. Ignored in Win32 */
    ,   RECLS_F_DEVICES                     =   0x00000008  /*!< Include devices in search. Not currently supported */
    ,   RECLS_F_TYPEMASK                    =   0x00000FFF
    ,   RECLS_F_DIR_PROGRESS                =   0x00001000  /*!< Reports each traversed directory to the callback function supplied to Recls_SearchFeedback() */
    ,   RECLS_F_RECURSIVE                   =   0x00010000  /*!< Searches given directory and all sub-directories */
    ,   RECLS_F_NO_FOLLOW_LINKS             =   0x00020000  /*!< Does not expand links */
    ,   RECLS_F_DIRECTORY_PARTS             =   0x00040000  /*!< Fills out the directory parts. Supported from version 1.1.1 onwards */
    ,   RECLS_F_DETAILS_LATER               =   0x00080000  /*!< Does not fill out anything other than the path. When specified to Recls_Stat(), and no types (RECLS_F_TYPEMASK) are specified, this will allow some details for a non-existant path to be elicited. */
    ,   RECLS_F_PASSIVE_FTP                 =   0x00100000  /*!< Passive mode in FTP. Supported from version 1.5.1 onwards */
    ,   RECLS_F_MARK_DIRS                   =   0x00200000  /*!< Marks the directories with a trailing slash. */
    ,   RECLS_F_ALLOW_REPARSE_DIRS          =   0x00400000  /*!< Allow Win32 reparse point directories to be examined (which can cause infinite loops). */
#if 0
    ,   RECLS_F_CALC_CHECKSUM               =   0x00800000
#endif /* 0 */
    ,   RECLS_F_CALLBACKS_STDCALL_ON_WIN32  =   0x01000000  /*!< Treats the callbacks as <b>stdcall</b> calling convention on Win32. Default is <b>cdecl</b>. This is useful for interfacing as delegates with .NET */
    ,   RECLS_F_USE_TILDE_ON_NO_SEARCHROOT  =   0x04000000  /*!< Interprets a NULL or empty searchRoot as the home directory, rather than the current directory. Supported from version 1.8.1 onwards. */
#if 0
    ,   RECLS_F_DIR_SIZE_IS_NUM_FILES       =   0x02000000  /*!< This causes the size of the directory to be the number of files contained within it, rather than being 0. */
#endif /* 0 */
    ,   RECLS_F_IGNORE_HIDDEN_ENTRIES_ON_WIN32   =   0x08000000  /*!< This causes hidden files to be ignored. Currently supported on Win32 only. */

#if !defined(FILES)
    ,   FILES = RECLS_F_FILES /*!< Equivalent to \link recls::RECLS_F_FILES RECLS_F_FILES\endlink. */
#endif /* !FILES */
#if !defined(DIRECTORIES)
    ,   DIRECTORIES = RECLS_F_DIRECTORIES /*!< Equivalent to \link recls::RECLS_F_DIRECTORIES RECLS_F_DIRECTORIES\endlink. */
#endif /* !DIRECTORIES */
#if !defined(LINKS)
    ,   LINKS = RECLS_F_LINKS /*!< Equivalent to \link recls::RECLS_F_LINKS RECLS_F_LINKS\endlink. */
#endif /* !LINKS */
#if !defined(DEVICES)
    ,   DEVICES = RECLS_F_DEVICES /*!< Equivalent to \link recls::RECLS_F_DEVICES RECLS_F_DEVICES\endlink. */
#endif /* !DEVICES */
#if !defined(TYPEMASK)
    ,   TYPEMASK = RECLS_F_TYPEMASK /*!< Equivalent to \link recls::RECLS_F_TYPEMASK RECLS_F_TYPEMASK\endlink. */
#endif /* !TYPEMASK */
#if !defined(DIR_PROGRESS)
    ,   DIR_PROGRESS = RECLS_F_DIR_PROGRESS /*!< Equivalent to \link recls::RECLS_F_DIR_PROGRESS RECLS_F_DIR_PROGRESS\endlink. */
#endif /* !DIR_PROGRESS */
#if !defined(RECURSIVE)
    ,   RECURSIVE = RECLS_F_RECURSIVE /*!< Equivalent to \link recls::RECLS_F_RECURSIVE RECLS_F_RECURSIVE\endlink. */
#endif /* !RECURSIVE */
#if !defined(NO_FOLLOW_LINKS)
    ,   NO_FOLLOW_LINKS = RECLS_F_NO_FOLLOW_LINKS /*!< Equivalent to \link recls::RECLS_F_NO_FOLLOW_LINKS RECLS_F_NO_FOLLOW_LINKS\endlink. */
#endif /* !NO_FOLLOW_LINKS */
#if !defined(DIRECTORY_PARTS)
    ,   DIRECTORY_PARTS = RECLS_F_DIRECTORY_PARTS /*!< Equivalent to \link recls::RECLS_F_DIRECTORY_PARTS RECLS_F_DIRECTORY_PARTS\endlink. */
#endif /* !DIRECTORY_PARTS */
#if !defined(DETAILS_LATER)
    ,   DETAILS_LATER = RECLS_F_DETAILS_LATER /*!< Equivalent to \link recls::RECLS_F_DETAILS_LATER RECLS_F_DETAILS_LATER\endlink. */
#endif /* !DETAILS_LATER */
#if !defined(PASSIVE_FTP)
    ,   PASSIVE_FTP = RECLS_F_PASSIVE_FTP /*!< Equivalent to \link recls::RECLS_F_PASSIVE_FTP RECLS_F_PASSIVE_FTP\endlink. */
#endif /* !PASSIVE_FTP */
#if !defined(MARK_DIRS)
    ,   MARK_DIRS = RECLS_F_MARK_DIRS /*!< Equivalent to \link recls::RECLS_F_MARK_DIRS RECLS_F_MARK_DIRS\endlink. */
#endif /* !MARK_DIRS */
#if !defined(ALLOW_REPARSE_DIRS)
    ,   ALLOW_REPARSE_DIRS = RECLS_F_ALLOW_REPARSE_DIRS /*!< Equivalent to \link recls::RECLS_F_ALLOW_REPARSE_DIRS RECLS_F_ALLOW_REPARSE_DIRS\endlink. */
#endif /* !ALLOW_REPARSE_DIRS */
#if 0
#if !defined(CALC_CHECKSUM)
    ,   CALC_CHECKSUM = RECLS_F_CALC_CHECKSUM /*!< Equivalent to \link recls::RECLS_F_CALC_CHECKSUM RECLS_F_CALC_CHECKSUM\endlink. */
#endif /* !CALC_CHECKSUM */
#endif /* 0 */
#if !defined(CALLBACKS_STDCALL_ON_WIN32)
    ,   CALLBACKS_STDCALL_ON_WIN32 = RECLS_F_CALLBACKS_STDCALL_ON_WIN32 /*!< Equivalent to \link recls::RECLS_F_CALLBACKS_STDCALL_ON_WIN32 RECLS_F_CALLBACKS_STDCALL_ON_WIN32\endlink. */
#endif /* !CALLBACKS_STDCALL_ON_WIN32 */
/*  ,   DIR_SIZE_IS_NUM_CHILDREN    =   RECLS_F_DIR_SIZE_IS_NUM_CHILDREN    */
#if !defined(F_USE_TILDE_ON_NO_SEARCHROOT)
    ,   USE_TILDE_ON_NO_SEARCHROOT = RECLS_F_USE_TILDE_ON_NO_SEARCHROOT /*!< Equivalent to \link recls::RECLS_F_USE_TILDE_ON_NO_SEARCHROOT RECLS_F_USE_TILDE_ON_NO_SEARCHROOT\endlink. */
#endif /* !RECLS_F_USE_TILDE_ON_NO_SEARCHROOT */
    ,   IGNORE_HIDDEN_ENTRIES_ON_WIN32 = RECLS_F_IGNORE_HIDDEN_ENTRIES_ON_WIN32 /*!< Equivalent to \link recls::RECLS_F_IGNORE_HIDDEN_ENTRIES_ON_WIN32 RECLS_F_IGNORE_HIDDEN_ENTRIES_ON_WIN32\endlink. */
#if !defined(IGNORE_HIDDEN_ENTRIES_ON_WIN32)
#endif /* !IGNORE_HIDDEN_ENTRIES_ON_WIN32 */
#if 0
#if !defined(RECLS_F_DIR_SIZE_IS_NUM_FILES)
    ,   DIR_SIZE_IS_NUM_FILES = RECLS_F_DIR_SIZE_IS_NUM_FILES /*!< Equivalent to \link recls::RECLS_F_DIR_SIZE_IS_NUM_FILES RECLS_F_DIR_SIZE_IS_NUM_FILES\endlink. */
#endif /* !RECLS_F_DIR_SIZE_IS_NUM_FILES */
#endif /* 0 */
};

/** \brief Selected Roots flags
 *
 * \ingroup group__recls
 *
 * These flags moderate the search behaviour of the Recls_GetSelectedRoots() function.
 *
 * \note The flags are ignored on UNIX, since the only root is '/'
 */
enum RECLS_ROOTS_FLAG
{
        RECLS_ROOTS_F_FIXED_DRIVES      =   0x0001  /*!< Include local fixed drives. */
    ,   RECLS_ROOTS_F_NETWORK_DRIVES    =   0x0002  /*!< Include network drives. */
    ,   RECLS_ROOTS_F_CDROM_DRIVES      =   0x0004  /*!< Include CD-ROM / DVD-ROM drives. */
    ,   RECLS_ROOTS_F_REMOVABLE_DRIVES  =   0x0008  /*!< Include removable drives. */
    ,   RECLS_ROOTS_F_RAM_DRIVES        =   0x0010  /*!< Include RAM drives. */

#ifdef __cplusplus
    ,   FixedDrives                     =   RECLS_ROOTS_F_FIXED_DRIVES      /*!< Include local fixed drives. Equivalent to \link recls::RECLS_ROOTS_F_FIXED_DRIVES RECLS_ROOTS_F_FIXED_DRIVES\endlink. */
    ,   NetworkDrives                   =   RECLS_ROOTS_F_NETWORK_DRIVES    /*!< Include network drives. Equivalent to \link recls::RECLS_ROOTS_F_NETWORK_DRIVES RECLS_ROOTS_F_NETWORK_DRIVES\endlink. */
    ,   CDRomDrives                     =   RECLS_ROOTS_F_CDROM_DRIVES      /*!< Include CD-ROM / DVD-ROM drives. Equivalent to \link recls::RECLS_ROOTS_F_CDROM_DRIVES RECLS_ROOTS_F_CDROM_DRIVES\endlink. */
    ,   RemovableDrives                 =   RECLS_ROOTS_F_REMOVABLE_DRIVES  /*!< Include removable drives. Equivalent to \link recls::RECLS_ROOTS_F_REMOVABLE_DRIVES RECLS_ROOTS_F_REMOVABLE_DRIVES\endlink. */
    ,   RamDrives                       =   RECLS_ROOTS_F_RAM_DRIVES        /*!< Include RAM drives. Equivalent to \link recls::RECLS_ROOTS_F_RAM_DRIVES RECLS_ROOTS_F_RAM_DRIVES\endlink. */
#endif /* __cplusplus */

#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
    ,   RECLS_F_FIXED_DRIVES            =   RECLS_ROOTS_F_FIXED_DRIVES
    ,   RECLS_F_NETWORK_DRIVES          =   RECLS_ROOTS_F_NETWORK_DRIVES
    ,   RECLS_F_CDROM_DRIVES            =   RECLS_ROOTS_F_CDROM_DRIVES
    ,   RECLS_F_REMOVABLE_DRIVES        =   RECLS_ROOTS_F_REMOVABLE_DRIVES
    ,   RECLS_F_RAM_DRIVES              =   RECLS_ROOTS_F_RAM_DRIVES
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */
};

#if !defined(__cplusplus) && \
    !defined(RECLS_DOCUMENTATION_SKIP_SECTION)
typedef enum RECLS_FLAG         RECLS_FLAG;
typedef enum RECLS_ROOTS_FLAG   RECLS_ROOTS_FLAG;
#endif /* !__cplusplus && !RECLS_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

/* #if defined(RECLS_PLATFORM_IS_WIN32)
# define RECLS_ROOT_NAME_LEN            (3)
#elif defined(RECLS_PLATFORM_IS_UNIX)
 */
#ifndef RECLS_COMPILER_IS_CH
/** \brief Structure containing information about a file-system root.
 *
 * \ingroup group__recls
 */
struct recls_root_t
{
    /** \brief The name of the root.
     *
     * \remarks For UNIX compilation, this is a two character nul-terminated
     *   array that always contains the string <code>"/"</code>. On Win32,
     *   this is a four character nul-terminated array that contains a drive
     *   specification, e.g. <code>"H:\\"</code>.
     */
    recls_char_t    name[1 + RECLS_ROOT_NAME_LEN];
};

# ifdef __cplusplus
typedef recls_root_t                    root_t;
# else /* ? __cplusplus */
typedef struct recls_root_t             recls_root_t;
# endif /* __cplusplus */
#endif /* !RECLS_COMPILER_IS_CH */

#ifndef RECLS_COMPILER_IS_CH
struct recls_fileinfo_t;
#endif /* !RECLS_COMPILER_IS_CH */

/** \brief Opaque type representing a file-system entry information.
 *
 * \ingroup group__recls
 */
typedef struct recls_fileinfo_t const   *recls_info_t;

/** \brief Opaque type representing a user-defined parameter to the process function.
 *
 * \ingroup group__recls
 */
typedef void                            *recls_process_fn_param_t;

/** \brief User-supplied process function, used by Recls_SearchProcess()
 *
 * \ingroup group__recls
 *
 * \param hEntry entry info structure
 * \param param the parameter passed to Recls_SearchProcess()
 *
 * \return A status to indicate whether to continue or cancel the processing
 * \retval 0 cancel the processing
 * \retval non-0 continue the processing
 */
typedef int (RECLS_CALLCONV_DEFAULT *hrecls_process_fn_t)(  recls_info_t                hEntry
                                                        ,   recls_process_fn_param_t    param);

/** \brief User-supplied process function, used by Recls_SearchFeedback(), which receives the
 * sub-directories traversed
 *
 * \ingroup group__recls
 *
 * \param dir The nul-terminated C-style string containing the sub-directory's full path
 * \param dirLen The number of characters in the \c dir param, not including the nul-terminator
 * \param param the parameter passed to Recls_SearchFeedback()
 *
 * \return A status to indicate whether to continue or cancel the processing
 * \retval 0 cancel the processing
 * \retval non-0 continue the processing
 */
typedef int (RECLS_CALLCONV_DEFAULT *hrecls_progress_fn_t)( recls_char_t const          *dir
                                                        ,   size_t                      dirLen
                                                        ,   recls_process_fn_param_t    param
                                                        ,   void                        *reserved0
                                                        ,   recls_uint32_t              reserved1);

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace typedefs
 */

#if !defined(RECLS_NO_NAMESPACE)
typedef recls_info_t                info_t;
typedef recls_process_fn_param_t    process_fn_param_t;
#endif /* !RECLS_NO_NAMESPACE */

/* /////////////////////////////////////////////////////////////////////////////
 * Functions
 */

/***************************************
 * File system
 */

/** \name File system functions
 *
 * \ingroup group__recls
 */
/** @{ */

#ifndef RECLS_COMPILER_IS_CH
/** \brief Retrieves the file-system roots for the current operating system.
 *
 * \ingroup group__recls
 *
 * It is used as follows:
 *
\htmlonly
<pre>
  recls_root_t  roots[26];
  size_t        cRoots  = Recls_GetRoots(&roots[0], 26);
  size_t        i;

  for(i = 0; i < cRoots; ++i)
  {
    puts(roots[i].name);
  }
</pre>
\endhtmlonly
 *
 * \param roots A pointer to an array of size \c cRoots. If NULL, the function returns the number of available roots.
 * \param cRoots The maximum number of roots to retrive in \c roots.
 *
 * \return The number of roots actually retrieved. If \c roots is NULL, then this is the current number of roots on
 * the current operating system. If \c roots is not NULL, then this is the number of roots written into \c roots, and
 * is not more than \c cRoots.
 */
RECLS_FNDECL(size_t)                Recls_GetRoots(     recls_root_t    *roots
                                                    ,   size_t          cRoots);

/** \brief Like Recls_GetRoots(), but only returns roots of selected disk types
 *
 * \ingroup group__recls
 *
 * \param roots A pointer to an array of size \c cRoots. If NULL, the function returns the number of available roots.
 * \param cRoots The maximum number of roots to retrive in \c roots.
 * \param flags Combination of the RECLS_ROOTS_FLAG enumeration. If 0, implies all available root types.
 *
 * \return The number of roots actually retrieved. If \c roots is NULL, then this is the current number of roots on
 * the current operating system. If \c roots is not NULL, then this is the number of roots written into \c roots, and
 * is not more than \c cRoots.
 */
RECLS_FNDECL(size_t)                Recls_GetSelectedRoots( recls_root_t    *roots
                                                        ,   size_t          cRoots
                                                        ,   recls_uint32_t  flags);

#endif /* !RECLS_COMPILER_IS_CH */

#if 0
/* New Recls1 API. */
RECLS_FNDECL(size_t)    Recls1_FileSystem_GetRoots(     recls_root_t    *roots
                                                    ,   size_t          cRoots
                                                    ,   recls_uint32_t  flags);
#endif /* 0 */


/** \brief Returns the symbol used to separate distinct path names in path name lists.
 *
 * \ingroup group__recls
 *
 * \note On UNIX this is \c "/"; on Win32 it is \c "\".
 *
 * It is used as follows:
 *
\htmlonly
<pre>
  puts(Recls_GetPathNameSeparator());
</pre>
\endhtmlonly
 */
RECLS_FNDECL(recls_char_t const*)   Recls_GetPathNameSeparator(void);

#if 0
/* New Recls1 API. */
RECLS_FNDECL(recls_char_t const*)   Recls1_FileSystem_GetPathNameSeparator(void);
#endif /* 0 */

/** \brief Returns the symbol used to separate the directory parts within paths.
 *
 * \ingroup group__recls
 *
 * \note On UNIX this is \c ":"; on Win32 it is \c ";".
 *
 * It is used as follows:
 *
\htmlonly
<pre>
  puts(Recls_GetPathSeparator());
</pre>
\endhtmlonly
 */
RECLS_FNDECL(recls_char_t const*)   Recls_GetPathSeparator(void);

#if 0
/* New Recls1 API. */
RECLS_FNDECL(recls_char_t const*)   Recls1_FileSystem_GetPathSeparator(void);
#endif /* 0 */

/** \brief Returns the wildcard symbol used to represent the "all files" for the current operating system.
 *
 * \ingroup group__recls
 *
 * \note On UNIX this is \c "*"; on Win32 it is \c "*.*".
 *
 * It is used as follows:
 *
\htmlonly
<pre>
  puts(Recls_GetWildcardsAll());
</pre>
\endhtmlonly
 */
RECLS_FNDECL(recls_char_t const*)   Recls_GetWildcardsAll(void);

#if 0
/* New Recls1 API. */
RECLS_FNDECL(recls_char_t const*)   Recls1_FileSystem_GetWildcardsAll(void);
#endif /* 0 */

/** @} */

/***************************************
 * Search control
 */

/** \name Search control functions
 *
 * \ingroup group__recls
 */
/** @{ */

/** \brief Searches a given directory for matching files of the given pattern
 *
 * \ingroup group__recls
 *
 * \param searchRoot The directory representing the root of the search. NULL
 *   or "" means the current directory is assumed, unless
 *   \link recls::RECLS_F_USE_TILDE_ON_NO_SEARCHROOT RECLS_F_USE_TILDE_ON_NO_SEARCHROOT\endlink
 *   is specified, in which case the calling identity's home directory is
 *   assumed.
 * \param pattern The search pattern, e.g. "*.c". NULL means "all files". ""
 *   means no files.
 * \param flags A combination of 0 or more
 *   \link recls::RECLS_FLAG RECLS_FLAG\endlink values.
 * \param phSrch Address of the search handle. This is set to NULL on
 *   failure.
 *
 * \return A status code indicating success/failure.
 * \retval RECLS_RC_NO_MORE_DATA No items matched the given search criteria.
 */
RECLS_FNDECL(recls_rc_t) Recls_Search(          recls_char_t const          *searchRoot
                                    ,           recls_char_t const          *pattern
                                    ,           recls_uint32_t              flags
                                    ,           hrecls_t                    *phSrch);

/** \brief Searches a given directory for matching files of the given pattern, calling back
 * on the given progress function pointer to inform the caller as each (sub-)directory
 * is traversed.
 *
 * \ingroup group__recls
 *
 * \param searchRoot The directory representing the root of the search
 * \param pattern The search pattern, e.g. "*.c"
 * \param flags A combination of 0 or more
 *   \link recls::RECLS_FLAG RECLS_FLAG\endlink values.
 * \param pfn The function that will be invoked for each directory traversed
 * \param param The caller-defined parameter that is passed to the callback function
 * \param phSrch Address of the search handle. This is set to NULL on failure
 *
 * \return A status code indicating success/failure
 */
RECLS_FNDECL(recls_rc_t) Recls_SearchFeedback(  recls_char_t const          *searchRoot
                                    ,           recls_char_t const          *pattern
                                    ,           recls_uint32_t              flags
                                    ,           hrecls_progress_fn_t        pfn
                                    ,           recls_process_fn_param_t    param
                                    ,           hrecls_t                    *phSrch);


#if 0
/* New Recls1 API. */
RECLS_FNDECL(recls_rc_t) Recls1_FileSystem_StartSearch( recls_char_t const          *searchRoot
                                                    ,   recls_char_t const          *pattern
                                                    ,   recls_uint32_t              flags
                                                    ,   Recls1_progress_fn_t        pfnProgress
                                                    ,   Recls1_process_fn_param_t   paramProgress
                                                    ,   Recls1_filter_fn_t          pfnFilter
                                                    ,   Recls1_filter_fn_param_t    paramFilter
                                                    ,   hrecls_t                    *phSrch);
#endif /* 0 */

/** \brief Searches a given directory for matching files of the given pattern, and processes them according to the given process function
 *
 * \ingroup group__recls
 *
 * \param searchRoot The directory representing the root of the search
 * \param pattern The search pattern, e.g. "*.c"
 * \param flags A combination of 0 or more
 *   \link recls::RECLS_FLAG RECLS_FLAG\endlink values.
 * \param pfn The processing function
 * \param param A caller-supplied parameter that is passed through to \c pfn on each invocation. The function can cancel the enumeration by returning 0
 *
 * \return A status code indicating success/failure
 *
 * \note Available from version 1.1 of the <b>recls</b> API
 */
RECLS_FNDECL(recls_rc_t) Recls_SearchProcess(   recls_char_t const          *searchRoot
                                            ,   recls_char_t const          *pattern
                                            ,   recls_uint32_t              flags
                                            ,   hrecls_process_fn_t         pfn
                                            ,   recls_process_fn_param_t    param);

/** \brief Closes the given search
 *
 * \ingroup group__recls
 *
 * \param hSrch Handle of the search to close. May not be NULL.
 */
RECLS_FNDECL(void) Recls_SearchClose(           hrecls_t                    hSrch);


#if 0
/* New Recls1 API. */
RECLS_FNDECL(recls_rc_t) Recls1_FileSystem_CloseSearch(hrecls_t             hSrch);
#endif /* 0 */


/** \brief Retrieves the information for the given path
 *
 * \ingroup group__recls
 *
 * \param path Absolute or relative path of the file system entry whose information is to be retrieved.
 * \param flags Flags to moderate the search. Only the type flags (flags & RECLS_F_TYPEMASK) and RECLS_F_DIRECTORY_PARTS are recognised
 * \param phEntry Pointer to receive entry info structure.
 *
 * \return Status code
 * \retval RECLS_RC_OK Position was advanced; search handle can be queried for details
 * \retval RECLS_RC_NO_MORE_DATA There was no file system entry that matched the \c path and \c flags criteria
 * \retval Any other status code indicates an error
 */
RECLS_FNDECL(recls_rc_t) Recls_Stat(            recls_char_t const          *path
                                            ,   recls_uint32_t              flags
                                            ,   recls_info_t                *phEntry);

#if 0
/* New Recls1 API. */
RECLS_FNDECL(recls_rc_t) Recls1_FileSystem_Stat(Recls1_char_t const         *path
                                            ,   Recls1_uint32_t             flags
                                            ,   Recls1_entry_t              *pentry);
#endif /* 0 */


/** @} */

/***************************************
 * Search enumeration
 */

/** \name Search enumeration functions
 *
 * \ingroup group__recls
 */
/** @{ */

/** \brief Advances the search one position
 *
 * \ingroup group__recls
 *
 * \param hSrch Handle of the search to close. May not be NULL.
 *
 * \return Status code
 * \retval RECLS_RC_OK Position was advanced; search handle can be queried for details
 * \retval RECLS_RC_NO_MORE_DATA There are no more items in the search
 * \retval Any other status code indicates an error
 */
RECLS_FNDECL(recls_rc_t) Recls_GetNext(         hrecls_t                    hSrch);

#if 0
/* New Recls1 API. */
RECLS_FNDECL(recls_rc_t) Recls1_Search_Next(    hrecls_t                    hSrch);
#endif /* 0 */


/** \brief Retrieves the information for the current search position
 *
 * \ingroup group__recls
 *
 * \param hSrch Handle of the search to close. May not be NULL.
 * \param phEntry Pointer to receive entry info structure.
 *
 * \return Status code
 * \retval RECLS_RC_OK Position was advanced; search handle can be queried for details
 * \retval RECLS_RC_NO_MORE_DATA There are no more items in the search
 * \retval Any other status code indicates an error
 */
RECLS_FNDECL(recls_rc_t) Recls_GetDetails(      hrecls_t                    hSrch
                                            ,   recls_info_t                *phEntry);

#if 0
/* New Recls1 API. */
#endif /* 0 */

/** \brief Advances the search one position, and retrieves the information for the new position
 *
 * \ingroup group__recls
 *
 * \param hSrch Handle of the search to close. May not be NULL.
 * \param phEntry Pointer to receive entry info structure.
 *
 * \return Status code
 * \retval RECLS_RC_OK Position was advanced; search handle can be queried for details
 * \retval Any other status code indicates an error
 */
RECLS_FNDECL(recls_rc_t) Recls_GetNextDetails(  hrecls_t                    hSrch
                                            ,   recls_info_t                *phEntry);

/** @} */

/***************************************
 * File entry information
 */

/** \name File entry information functions
 *
 * \ingroup group__recls
 */
/** @{ */

/** \brief Releases the resources associated with an entry info structure.
 *
 * \ingroup group__recls
 *
 * \param hEntry The info entry structure handle. Must not be NULL.
 */
RECLS_FNDECL(void) Recls_CloseDetails(          recls_info_t                hEntry);

/** \brief Copies an entry info structure.
 *
 * \ingroup group__recls
 *
 * \param hEntry The info entry structure.
 * \param phEntry Address to receive a copy of the info entry structure. May not be NULL.
 *
 * \return Status code
 * \retval RECLS_RC_OK Entry was generated.
 * \retval Any other status code indicates an error
 */
RECLS_FNDECL(recls_rc_t) Recls_CopyDetails(     recls_info_t                hEntry
                                            ,   recls_info_t                *phEntry);

/** \brief Reports on the number of outstanding (i.e. in client code) file entry info structures
 *
 * \ingroup group__recls
 *
 * \param hSrch Handle of the search to close. May not be NULL.
 * \param count Pointer to an integer variable to receive the result.
 *
 * \return Status code
 * \retval RECLS_RC_OK Information was retrieved.
 * \retval Any other status code indicates an error
 */
RECLS_FNDECL(recls_rc_t) Recls_OutstandingDetails(hrecls_t                  hSrch
                                            ,   recls_uint32_t              *count);

/** @} */

/***************************************
 * Error handling
 */

/** \name Error handling functions
 *
 * \ingroup group__recls
 */
/** @{ */

/** \brief Returns the last error code associated with the given search handle
 *
 * \ingroup group__recls
 *
 * \param hSrch Handle of the search to close. May not be NULL.
 *
 * \return The last error code for the search handle
 */
RECLS_FNDECL(recls_rc_t) Recls_GetLastError(    hrecls_t                    hSrch);

/** \brief Gets the error string representing the given error
 *
 * \ingroup group__recls
 *
 * \param rc The error code
 * \param buffer Pointer to character buffer in which to write the error. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for the error.
 */
RECLS_FNDECL(size_t) Recls_GetErrorString(      /* [in] */ recls_rc_t       rc
                                    ,           /* [in] */ recls_char_t     *buffer
                                    ,           /* [in] */ size_t           cchBuffer);

/** \brief Gets the error string representing the current error associated with the given search handle
 *
 * \ingroup group__recls
 *
 * \param hSrch Handle of the search to close. May not be NULL.
 * \param buffer Pointer to character buffer in which to write the error. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the error.
 */
RECLS_FNDECL(size_t) Recls_GetLastErrorString(  /* [in] */ hrecls_t         hSrch
                                        ,       /* [in] */ recls_char_t     *buffer
                                        ,       /* [in] */ size_t           cchBuffer);

/** @} */

/***************************************
 * Property elicitation
 */

/** \name Property elicitation functions
 *
 * \ingroup group__recls
 */
/** @{ */

/** \brief Retrieves the full path of the given entry recls_fileinfo_t
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param buffer Pointer to character buffer in which to write the path. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the path.
 */
RECLS_FNDECL(size_t) Recls_GetPathProperty(     recls_info_t                hEntry
                                        ,       recls_char_t                *buffer
                                        ,       size_t                      cchBuffer);

/** \brief Retrieves the path of the given entry recls_fileinfo_t, relative to the search directory
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param buffer Pointer to character buffer in which to write the path. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the path.
 */
RECLS_FNDECL(size_t) Recls_GetSearchRelativePathProperty(   recls_info_t    hEntry
                                                    ,       recls_char_t    *buffer
                                                    ,       size_t          cchBuffer);

/** \brief Retrieves the directory of the given entry recls_fileinfo_t
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param buffer Pointer to character buffer in which to write the directory. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the directory.
 */
RECLS_FNDECL(size_t) Recls_GetDirectoryProperty(recls_info_t                hEntry
                                            ,   recls_char_t                *buffer
                                            ,   size_t                      cchBuffer);

/** \brief Retrieves the directory (and drive) of the given entry recls_fileinfo_t
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param buffer Pointer to character buffer in which to write the directory. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the directory.
 *
 * \note On systems that do not have a drive, this function behaves identically to Recls_GetDirectoryProperty()
 */
RECLS_FNDECL(size_t) Recls_GetDirectoryPathProperty(    recls_info_t        hEntry
                                                    ,   recls_char_t        *buffer
                                                    ,   size_t              cchBuffer);

/** \brief Retrieves the directory (and drive) of the given entry recls_fileinfo_t that was used in the search resulting in the given entry
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param buffer Pointer to character buffer in which to write the directory. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the directory.
 */
RECLS_FNDECL(size_t) Recls_GetSearchDirectoryProperty(  recls_info_t        hEntry
                                                    ,   recls_char_t        *buffer
                                                    ,   size_t              cchBuffer);

/** \brief Retrieves the UNC drive property of the given entry recls_fileinfo_t
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param buffer Pointer to character buffer in which to write the directory. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the directory.
 *
 * \note On operating systems that do not support UNC, this function always returns 0, and writes the empty string into buffer (when non-NULL)
 */
RECLS_FNDECL(size_t) Recls_GetUNCDriveProperty(         recls_info_t        hEntry
                                                    ,   recls_char_t        *buffer
                                                    ,   size_t              cchBuffer);

/** \brief Retrieves the file (filename + extension) of the given entry recls_fileinfo_t
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param buffer Pointer to character buffer in which to write the file. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the file.
 */
RECLS_FNDECL(size_t) Recls_GetFileProperty(     recls_info_t                hEntry
                                        ,       recls_char_t                *buffer
                                        ,       size_t                      cchBuffer);

/** \brief Retrieves the short version of the file of the given entry recls_fileinfo_t
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param buffer Pointer to character buffer in which to write the file. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the file.
 *
 * \note On systems where there is no concept of a short name, this function behaves exactly as Recls_GetFileProperty()
 */
RECLS_FNDECL(size_t) Recls_GetShortFileProperty(recls_info_t                hEntry
                                            ,   recls_char_t                *buffer
                                            ,   size_t                      cchBuffer);

/** \brief Retrieves the filename (not including extension, if any) of the given entry recls_fileinfo_t
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param buffer Pointer to character buffer in which to write the filename. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the filename.
 */
RECLS_FNDECL(size_t) Recls_GetFileNameProperty( recls_info_t                hEntry
                                            ,   recls_char_t                *buffer
                                            ,   size_t                      cchBuffer);

/** \brief Retrieves the file extension of the given entry recls_fileinfo_t
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param buffer Pointer to character buffer in which to write the extension. If NULL, the function returns the number of characters required.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the extension.
 */
RECLS_FNDECL(size_t) Recls_GetFileExtProperty(  recls_info_t                hEntry
                                            ,   recls_char_t                *buffer
                                            ,   size_t                      cchBuffer);

/** \brief Retrieves a directory part of the given entry recls_fileinfo_t
 *
 * \ingroup group__recls
 *
 * \param hEntry The entry recls_fileinfo_t. Cannot be NULL
 * \param part The part requested. If -1, then the function returns the number of parts
 * \param buffer Pointer to character buffer in which to write the extension. If NULL, the function returns the number of characters required. Ignored if part is -1.
 * \param cchBuffer Number of character spaces in \c buffer. Ignored if \c buffer is NULL or part is -1.
 *
 * \return If \c part is -1, returns the number of parts. Otherwise, The number of characters written to the buffer, or required for, the extension.
 *
 * \note The behaviour is undefined if part is outside the range of parts.
 */
RECLS_FNDECL(size_t) Recls_GetDirectoryPartProperty(recls_info_t            hEntry
                                            ,   int                         part
                                            ,   recls_char_t                *buffer
                                            ,   size_t                      cchBuffer);


/** \brief Returns non-zero if the file entry is read-only.
 *
 * \ingroup group__recls
 *
 * \param hEntry The file entry info structure to test. May not be NULL
 * \retval true file entry is read-only
 * \retval false file entry is not read-only
 *
 * \note There is no error return
 */
RECLS_FNDECL(recls_bool_t) Recls_IsFileReadOnly(recls_info_t                hEntry);

/** \brief Returns non-zero if the file entry represents a directory.
 *
 * \ingroup group__recls
 *
 * \param hEntry The file entry info structure to test. May not be NULL
 * \retval true file entry is a directory
 * \retval false file entry is not directory
 *
 * \note There is no error return
 */
RECLS_FNDECL(recls_bool_t) Recls_IsFileDirectory(recls_info_t               hEntry);

/** \brief Returns non-zero if the file entry represents a link.
 *
 * \ingroup group__recls
 *
 * \param hEntry The file entry info structure to test. May not be NULL
 * \retval true file entry is a link
 * \retval false file entry is not link
 *
 * \note There is no error return
 */
RECLS_FNDECL(recls_bool_t) Recls_IsFileLink(    recls_info_t                hEntry);

/** \brief Returns non-zero if the file entry exists "now".
 *
 * \ingroup group__recls
 *
 * \param hEntry The file entry info structure to test. May not be NULL
 * \retval true file entry exists in the file-system at the time of call.
 * \retval false file entry does not exist in the file-system at the time of call
 *
 * \note There is no error return.
 */
RECLS_FNDECL(recls_bool_t) Recls_DoesEntryExist(recls_info_t                hEntry);

/** \brief Returns non-zero if the file entry's path is UNC.
 *
 * \ingroup group__recls
 *
 * \param hEntry The file entry info structure to test. May not be NULL
 * \retval true file entry's path is UNC
 * \retval false file entry's path is UNC
 *
 * \note There is no error return
 */
RECLS_FNDECL(recls_bool_t) Recls_IsFileUNC(     recls_info_t                hEntry);

/** \brief Acquires the size of the file entry.
 *
 * \ingroup group__recls
 *
 * \param hEntry The file entry info structure to test. May not be NULL
 * \param size Pointer to the location in which to store the size
 *
 * \note There is no error return. File system entries that do not have a meaningful size will be given a notional size of 0.
 */
RECLS_FNDECL(void)         Recls_GetSizeProperty(   recls_info_t            hEntry
                                                ,   recls_filesize_t        *size);

/** \brief Returns the time the file was created.
 *
 * \ingroup group__recls
 */
RECLS_FNDECL(recls_time_t) Recls_GetCreationTime(recls_info_t               hEntry);

/** \brief Returns the time the file was last modified.
 *
 * \ingroup group__recls
 */
RECLS_FNDECL(recls_time_t) Recls_GetModificationTime(recls_info_t           hEntry);

/** \brief Returns the time the file was last accessed.
 *
 * \ingroup group__recls
 */
RECLS_FNDECL(recls_time_t) Recls_GetLastAccessTime(recls_info_t             hEntry);

/** \brief Returns the time the file status was last changed.
 *
 * \ingroup group__recls
 */
RECLS_FNDECL(recls_time_t) Recls_GetLastStatusChangeTime(recls_info_t       hEntry);

#if 0
/** \brief Returns the checksum value of the file.
 *
 * \ingroup group__recls
 *
 * \note This will return 0 if the entry is not a file, or its contents could
 * not be read, or RECLS_F_CALC_CHECKSUM was not specified.
 */
RECLS_FNDECL(recls_uint32_t) Recls_GetChecksum(         recls_info_t        hEntry);
#endif /* 0 */

/** @} */

/***************************************
 * Extended API functions
 */

/** \name Extended API functions
 *
 * \ingroup group__recls
 */
/** @{ */

/** \brief Determines whether the given directory is empty
 *
 * \ingroup group__recls
 *
 * \param dir The directory to assess
 */
RECLS_FNDECL(recls_bool_t) Recls_IsDirectoryEmpty(recls_char_t const        *dir);

/** \brief Determines whether the given directory entry is empty
 *
 * \ingroup group__recls
 *
 * \param hEntry The directory entry to assess
 */
RECLS_FNDECL(recls_bool_t) Recls_IsDirectoryEntryEmpty(recls_info_t         hEntry);

/** \brief Calculates the number of bytes in files in the given directory
 *    and all its sub-directories.
 *
 * \ingroup group__recls
 *
 * \param dir The directory to assess
 */
RECLS_FNDECL(recls_filesize_t) Recls_CalcDirectorySize(recls_char_t const   *dir);

/** \brief Calculates the number of bytes in files in the given directory
 *    entry and all its sub-directories.
 *
 * \ingroup group__recls
 *
 * \param hEntry The directory entry to assess
 */
RECLS_FNDECL(recls_filesize_t) Recls_CalcDirectoryEntrySize(recls_info_t    hEntry);


/** \brief Calculates the number of bytes in files in the given directory
 *    and all its sub-directories, calling back on the given progress
 *    function pointer to inform the caller as each (sub-)directory is
 *    traversed.
 *
 * \param flags Ignores RECLS_F_DIRECTORIES, and always adds RECLS_F_FILES. Listens to selected others.
 * \param pfn The function that will be invoked for each directory traversed
 * \param param The caller-defined parameter that is passed to the callback function
 *
 * \return A status code indicating success/failure
 *
 */
RECLS_FNDECL(recls_filesize_t) Recls_CalcDirectorySizeFeedback( recls_char_t const          *dir
                                                            ,   recls_uint32_t              flags
                                                            ,   hrecls_progress_fn_t        pfn
                                                            ,   recls_process_fn_param_t    param);

/** @} */

/***************************************
 * Path functions
 */

/** \name Path functions
 *
 * \ingroup group__recls
 */
/** @{ */

/** \brief Combines two paths into one.
 *
 * \ingroup group__recls
 *
 * This function takes two path fragments and combines them to form a single path.
 *
 * \param path1 The left-most path.
 * \param path2 The right-most path.
 * \param result Pointer to character buffer in which to write the resultant path. If NULL, the function returns the number of characters required.
 * \param cchResult Number of character spaces in \c result. Ignored if \c result is NULL.
 *
 * \return The number of characters written to the buffer, or required for, the path.
 *
 * \remarks If \c path2 is an absolute path, \c path1 is ignored and \c result contains characters copied from \c path2.
 */
RECLS_FNDECL(size_t) Recls_CombinePaths(        recls_char_t const          *path1
                                            ,   recls_char_t const          *path2
                                            ,   recls_char_t                *result
                                            ,   size_t                      cchResult);

/** \brief Determines the relative path between two paths
 *
 * \ingroup group__recls
 *
 * \param origin The path against which the relativity of \c target will be evaluated.
 * \param target The path.whose relatively (against \c origin) will be evaluated.
 * \param result Pointer to character buffer in which to write the resultant path. If NULL, the function returns the number of characters required.
 * \param cchResult Number of character spaces in \c result. Ignored if \c result is NULL.
 *
 * \remarks On operating systems that have separate drives, 
 */
RECLS_FNDECL(size_t) Recls_DeriveRelativePath(  recls_char_t const          *origin
                                            ,   recls_char_t const          *target
                                            ,   recls_char_t                *result
                                            ,   size_t                      cchResult);

/** \brief Prepares the path for display into a fixed maximum width field.
 *
 * \ingroup group__recls
 *
 * \param path The path to be squeezed.
 * \param result Pointer to character buffer in which to write the resultant path. If NULL, the function returns the number of characters required.
 * \param cchResult Number of character spaces in \c result. Ignored if \c result is NULL.
 *
 * \remarks On operating systems that have separate drives, 
 */
RECLS_FNDECL(size_t) Recls_SqueezePath(         recls_char_t const          *path
                                            ,   recls_char_t                *result
                                            ,   size_t                      cchResult);

/** @} */

/* /////////////////////////////////////////////////////////////////////////////
 * Namespace
 */

#if !defined(RECLS_NO_NAMESPACE)
} /* namespace recls */
#endif /* !RECLS_NO_NAMESPACE */

#ifdef __cplusplus

# ifndef _STLSOFT_NO_NAMESPACE
namespace stlsoft
{
# endif /* !_STLSOFT_NO_NAMESPACE */

# if defined(RECLS_NO_NAMESPACE)
inline recls_char_t const *c_str_ptr(recls_root_t const &root)
# else /* ? RECLS_NO_NAMESPACE */
inline ::recls::recls_char_t const *c_str_ptr(::recls::recls_root_t const &root)
# endif /* RECLS_NO_NAMESPACE */
{
    return root.name;
}

#ifdef RECLS_CHAR_TYPE_IS_CHAR
inline ::recls::recls_char_t const *c_str_ptr_a(::recls::recls_root_t const &root)
{
    return c_str_ptr(root);
}
#endif /* RECLS_CHAR_TYPE_IS_CHAR */

#ifdef RECLS_CHAR_TYPE_IS_WCHAR
inline ::recls::recls_char_t const *c_str_ptr_w(::recls::recls_root_t const &root)
{
    return c_str_ptr(root);
}
#endif /* RECLS_CHAR_TYPE_IS_WCHAR */


# ifndef _STLSOFT_NO_NAMESPACE
} /* namespace stlsoft */
# endif /* !_STLSOFT_NO_NAMESPACE */

#endif /* __cplusplus */

/* /////////////////////////////////////////////////////////////////////////////
 * Platform-specific includes
 */

/** \def RECLS_PLATFORM_API_WIN32
 *
 * \brief Defined if Win32 platform-specific extensions are in use.
 */
/** \def RECLS_PLATFORM_API_UNIX
 *
 * \brief Defined if UNIX platform-specific extensions are in use.
 */

#ifdef RECLS_PLATFORM_API_WIN32
# undef RECLS_PLATFORM_API_WIN32
#endif /* RECLS_PLATFORM_API_WIN32 */

#ifdef RECLS_PLATFORM_API_UNIX
# undef RECLS_PLATFORM_API_UNIX
#endif /* RECLS_PLATFORM_API_UNIX */

#if !defined(RECLS_PURE_API)
# if defined(RECLS_PLATFORM_IS_WIN32)
#  include <recls/win32.h>
#  define RECLS_PLATFORM_API_WIN32
# elif defined(RECLS_PLATFORM_IS_UNIX)
#  include <recls/unix.h>
#  define RECLS_PLATFORM_API_UNIX
# else
#  error Platform not recognised
# endif /* platform */
#endif /* !RECLS_PURE_API */

/* /////////////////////////////////////////////////////////////////////////////
 * Inclusion
 */

#ifdef RECLS_CF_PRAGMA_ONCE_SUPPORT
# pragma once
#endif /* RECLS_CF_PRAGMA_ONCE_SUPPORT */

/* /////////////////////////////////////////////////////////////////////////////
 * Documentation
 */

#ifdef RECLS_DOCUMENTATION_SKIP_SECTION
# include <recls/internal/doc/includes.h>
#endif /* RECLS_DOCUMENTATION_SKIP_SECTION */

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* !RECLS_INCL_RECLS_H_RECLS */

/* ////////////////////////////////////////////////////////////////////////// */
