////////////////////////////////////////////////////////////////////////////////
// DirectoryPartsCollection.h : Declaration of the DirectoryPartsCollection
//
// Updated: 14th June 2006

#ifndef __DIRECTORYPARTSCOLLECTION_H_
#define __DIRECTORYPARTSCOLLECTION_H_

#include "resource.h"       // main symbols

////////////////////////////////////////////////////////////////////////////////

struct dirparts_proxy_traits
{
    static CComVariant make_value(const recls_strptrs_t &ptr)
    {
        CComVariant var;

        var.vt      =   VT_BSTR;
# ifdef RECLS_CHAR_TYPE_IS_CHAR
        var.bstrVal =   ::SysAllocStringLen(winstl::a2w(ptr.begin, ptr.end - ptr.begin), ptr.end - ptr.begin);
# elif defined(RECLS_CHAR_TYPE_IS_WCHAR)
        var.bstrVal =   ::SysAllocStringLen(ptr.begin, ptr.end - ptr.begin);
# endif /* RECLS_CHAR_TYPE_IS_WCHAR */

        return var;
    }
};

class _Copy_VARIANT
{
public:
    static HRESULT copy(VARIANT* p1, VARIANT const *p2)
    {
        return ::VariantCopy(p1, const_cast<VARIANT*>(p2));
    }
    static void init(VARIANT* p)
    {
        p->vt = VT_EMPTY;
    }
    static void destroy(VARIANT* p)
    {
        ::VariantClear(p);
    }
};


typedef stlsoft::proxy_sequence <   const recls_strptrs_t
                                ,   CComVariant
                                ,   dirparts_proxy_traits
                                >               directory_parts_type;

typedef CComEnumOnSTL           <   IEnumVARIANT
                                ,   &IID_IEnumVARIANT
                                ,   VARIANT
                                ,   _Copy_VARIANT
                                ,   directory_parts_type
                                >               Enumerator_t;

typedef ICollectionOnSTLImpl    <   IDirectoryPartsCollection
                                ,   directory_parts_type
                                ,   VARIANT
                                ,   _Copy_VARIANT
                                ,   Enumerator_t
                                >               Collection_t;

/////////////////////////////////////////////////////////////////////////////
// DirectoryPartsCollection

class ATL_NO_VTABLE DirectoryPartsCollection
    : public CComObjectRootEx<CComMultiThreadModelNoCS>
    , public IDispatchImpl<Collection_t, &IID_IDirectoryPartsCollection, &LIBID_RECLS_COMLib>
    , public atlstl::SupportErrorInfoImpl<&IID_IDirectoryPartsCollection>
{
private:
    typedef IDispatchImpl<Collection_t, &IID_IDirectoryPartsCollection, &LIBID_RECLS_COMLib> parent_class_type;

public:
    DirectoryPartsCollection()
        : m_punkOwner(NULL)
    {}
    ~DirectoryPartsCollection() stlsoft_throw_0()
    {
        if(NULL != m_punkOwner)
        {
            m_punkOwner->Release();
        }
    }

    HRESULT Init(LPUNKNOWN punkOwner, directory_parts_type &parts)
    {
        m_punkOwner = punkOwner;

        if(NULL != m_punkOwner)
        {
            m_punkOwner->AddRef();
        }

        m_coll = parts;

        return S_OK;
    }

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(DirectoryPartsCollection)
    COM_INTERFACE_ENTRY(IDirectoryPartsCollection)
    COM_INTERFACE_ENTRY(IDispatch)
    COM_INTERFACE_ENTRY(ISupportErrorInfo)
END_COM_MAP()

// IDirectoryPartsCollection
public:

// Members
private:
    LPUNKNOWN   m_punkOwner;
};

#endif //__DIRECTORYPARTSCOLLECTION_H_

////////////////////////////////////////////////////////////////////////////////
