////////////////////////////////////////////////////////////////////////////////
// FileEntry.cpp : Implementation of FileEntry
//
// Created: 27th September 2003 
// Updated: 15th January 2007
//

#include "stdafx.h"
#include "recls.com.h"
#include "FileEntry.h"
#include "DirectoryPartsCollection.h"

#include <winstl/filesystem/file_path_buffer.hpp>

/////////////////////////////////////////////////////////////////////////////
// FileEntry

FileEntry::FileEntry()
    : m_entry(NULL)
{
	;
}

FileEntry::~FileEntry() stlsoft_throw_0()
{
	;

    if(NULL != m_entry)
    {
        Recls_CloseDetails(m_entry);
    }
}

void FileEntry::SetVoid(void const *pv)
{
    m_entry = (info_t)pv;
}

////////////////////////////////////////////////////////////////////////////////
// IFileEntry

HRESULT FileEntry::GetStringProperty_(size_t (RECLS_CALLCONV_DEFAULT *pfn)(info_t, recls_char_t *, size_t), BSTR *pVal)
{
    HRESULT hr;

    if(NULL == pVal)
    {
        hr = E_POINTER;
    }
    else if(NULL == m_entry)
    {
        *pVal = NULL;

        hr = DISP_E_BADCALLEE;
    }
    else
    {
        size_t  cch = pfn(m_entry, NULL, 0);

        *pVal = ::SysAllocStringLen(NULL, cch);

        if(NULL == *pVal)
        {
            hr = E_OUTOFMEMORY;
        }
        else
        {
#ifdef RECLS_CHAR_TYPE_IS_CHAR
            typedef winstl::basic_file_path_buffer<recls_char_t, comstl::task_allocator<recls_char_t> > file_path_buffer;

            file_path_buffer    buffer;

            ATLASSERT(::lstrlen(m_entry->path.begin) <= buffer.size());

            pfn(m_entry, &buffer[0], cch);

            ::MultiByteToWideChar(0, 0, &buffer[0], cch, *pVal, cch);

#elif defined(RECLS_CHAR_TYPE_IS_WCHAR)
            pfn(m_entry, *pVal, cch);
#endif /* RECLS_CHAR_TYPE_IS_WCHAR */

            hr = S_OK;
        }
    }

    return hr;
}

STDMETHODIMP FileEntry::get_Path(BSTR *pVal)
{
    return GetStringProperty_(Recls_GetPathProperty, pVal);
}

STDMETHODIMP FileEntry::get_Drive(/*[out, retval]*/ OLECHAR *pVal)
{
    return (pVal == 0) ? E_POINTER : (*pVal = m_entry->drive, S_OK);
}

STDMETHODIMP FileEntry::get_Directory(BSTR *pVal)
{
    return GetStringProperty_(Recls_GetDirectoryProperty, pVal);
}

STDMETHODIMP FileEntry::get_DirectoryPath(BSTR *pVal)
{
    return GetStringProperty_(Recls_GetDirectoryPathProperty, pVal);
}

STDMETHODIMP FileEntry::get_DirectoryParts(LPUNKNOWN *pVal)
{
    // Create an instance of IEnumVARIANT from the directory parts here.
    typedef CComObject<DirectoryPartsCollection>    DirPartsColl_t;

    DirPartsColl_t  *pcoll;
    HRESULT         hr = DirPartsColl_t::CreateInstance(&pcoll);

    if(SUCCEEDED(hr))
    {
        directory_parts_type    dirParts(m_entry->directoryParts.begin, m_entry->directoryParts.end);

        pcoll->AddRef();

        hr = pcoll->Init(static_cast<IFileEntry*>(this), dirParts);

        if(SUCCEEDED(hr))
        {
            hr = pcoll->QueryInterface(pVal);
        }

        pcoll->Release();
    }

    return hr;
}

STDMETHODIMP FileEntry::get_File(BSTR *pVal)
{
    return GetStringProperty_(Recls_GetFileProperty, pVal);
}

STDMETHODIMP FileEntry::get_ShortFile(BSTR *pVal)
{
    return GetStringProperty_(Recls_GetShortFileProperty, pVal);
}

STDMETHODIMP FileEntry::get_FileName(BSTR *pVal)
{
    return GetStringProperty_(Recls_GetFileNameProperty, pVal);
}

STDMETHODIMP FileEntry::get_FileExt(BSTR *pVal)
{
    return GetStringProperty_(Recls_GetFileExtProperty, pVal);
}

HRESULT FileEntry::GetTimeProperty_(recls_time_t recls_fileinfo_t::*pm, DATE *pVal, char const *methodName)
{
    HRESULT hr;

    if(NULL == pVal)
    {
        hr = E_POINTER;
    }
    else
    {
		FILETIME	ftLocal;
        UDATE		ud;

		::FileTimeToLocalFileTime(&(m_entry->*pm), &ftLocal);

        if(!::FileTimeToSystemTime(&ftLocal, &ud.st))
		{
			pantheios::log_ERROR("FileEntry::", methodName, ": failed to convert time");

			hr = E_FAIL;
		}
		else
		{
	        hr = ::VarDateFromUdate(&ud, 0, pVal);
		}
    }

    return hr;
}

STDMETHODIMP FileEntry::get_CreationTime(DATE *pVal)
{
	pantheios::log_DEBUG("FileEntry::CreationTime");

    return GetTimeProperty_(&recls_fileinfo_t::creationTime, pVal, "CreationTime");
}

STDMETHODIMP FileEntry::get_ModificationTime(DATE *pVal)
{
	pantheios::log_DEBUG("FileEntry::ModificationTime");

    return GetTimeProperty_(&recls_fileinfo_t::modificationTime, pVal, "ModificationTime");
}

STDMETHODIMP FileEntry::get_LastAccessTime(DATE *pVal)
{
	pantheios::log_DEBUG("FileEntry::LastAccessTime");

    return GetTimeProperty_(&recls_fileinfo_t::lastAccessTime, pVal, "LastAccessTime");
}

STDMETHODIMP FileEntry::get_LastStatusChangeTime(DATE *pVal)
{
	pantheios::log_DEBUG("FileEntry::LastStatusChangeTime");

    return GetTimeProperty_(&recls_fileinfo_t::modificationTime, pVal, "LastStatusChangeTime");
}

STDMETHODIMP FileEntry::get_Size(long *pVal)
{
	pantheios::log_DEBUG("FileEntry::Size");

    HRESULT hr;

    if(NULL == pVal)
    {
        hr = E_POINTER;
    }
    else
    {
        recls_filesize_t    size;

        Recls_GetSizeProperty(m_entry, &size);

        *pVal = (long)size.LowPart;

        hr = S_OK;
    }

    return hr;
}

STDMETHODIMP FileEntry::get_IsReadOnly(BOOL *pVal)
{
    return (NULL == pVal) ? E_POINTER : ((*pVal = (Recls_IsFileReadOnly(m_entry) ? VARIANT_TRUE : VARIANT_FALSE)), S_OK);
}

STDMETHODIMP FileEntry::get_IsDirectory(BOOL *pVal)
{
    return (NULL == pVal) ? E_POINTER : ((*pVal = (Recls_IsFileDirectory(m_entry) ? VARIANT_TRUE : VARIANT_FALSE)), S_OK);
}

////////////////////////////////////////////////////////////////////////////////
// IFileEntry2

STDMETHODIMP FileEntry::get_UNCDrive(BSTR *pVal)
{
    return GetStringProperty_(Recls_GetUNCDriveProperty, pVal);
}

STDMETHODIMP FileEntry::get_IsLink(BOOL *pVal)
{
    return (NULL == pVal) ? E_POINTER : ((*pVal = (Recls_IsFileLink(m_entry) ? VARIANT_TRUE : VARIANT_FALSE)), S_OK);
}

STDMETHODIMP FileEntry::get_IsUNC(BOOL *pVal)
{
    return (NULL == pVal) ? E_POINTER : ((*pVal = (Recls_IsFileUNC(m_entry) ? VARIANT_TRUE : VARIANT_FALSE)), S_OK);
}

////////////////////////////////////////////////////////////////////////////////
// IFileEntry3

STDMETHODIMP FileEntry::get_SearchDirectory(BSTR *pVal)
{
    return GetStringProperty_(Recls_GetSearchDirectoryProperty, pVal);
}

////////////////////////////////////////////////////////////////////////////////
// IFileEntry3

STDMETHODIMP FileEntry::get_SearchRelativePath(BSTR *pVal)
{
    return GetStringProperty_(Recls_GetSearchRelativePathProperty, pVal);
}

////////////////////////////////////////////////////////////////////////////////
