/* /////////////////////////////////////////////////////////////////////////////
 * File:        FileSearch.java
 *
 * Purpose:     Implementation of the org.recls.FileSearch class
 *
 * Created      3rd June 2004
 * Updated:     16th June 2006
 *
 * Home:        http://recls.org/
 *
 * Copyright (c) 2004-2006, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


package org.recls;

import java.io.File;
import java.util.Enumeration;

/** This class represents a recursive search
 */
public class FileSearch
	extends Search
    implements Enumeration
{
/// Constants
    public static final int RECLS_ROOTS_F_FIXED_DRIVES      =   0x0001;	/*!< Include local fixed drives. */
    public static final int RECLS_ROOTS_F_NETWORK_DRIVES    =   0x0002; /*!< Include network drives. */
    public static final int RECLS_ROOTS_F_CDROM_DRIVES      =   0x0004; /*!< Include CD-ROM / DVD-ROM drives. */
    public static final int RECLS_ROOTS_F_REMOVABLE_DRIVES  =   0x0008; /*!< Include removable drives. */
    public static final int RECLS_ROOTS_F_RAM_DRIVES        =   0x0010; /*!< Include RAM drives. */

/// Construction
    /** Creates a new enumerable search */
    public FileSearch(String searchRoot, String pattern, int flags) throws ReclsException
    {
        super(initialise(searchRoot, pattern, flags));
    }

/// Class methods
	/** Returns an Entry representing the given path, moderated by the given flags.
	 */
	public native static Entry stat(String path, int flags);

	/** Returns an Entry representing the given path, moderated by the given flags.
	 *
	 * @deprecated Use stat() instead.
	 */
	public static Entry Stat(String path, int flags)
	{
		return stat(path, flags);
	}

	/** Returns an Entry representing the given path.
	 */
	public static Entry stat(String path)
	{
		return stat(path, 0);
	}

	/** Returns an array of the roots for the current system.
	 */
	public native static String[] getRoots();

	/** Returns an array of roots for the current system matching the
	 * given flags.
	 */
	public native static String[] getSelectedRoots(int flags);

	public native static String combinePaths(String path1, String path2);

	public native static String deriveRelativePath(String origin, String target);

	public native static String squeezePath(String path, int width);

/// Native method implementations
    private native static int      initialise(String searchRoot, String pattern, int flags) throws ReclsException;
}

/* ////////////////////////////////////////////////////////////////////////// */
