/* /////////////////////////////////////////////////////////////////////////////
 * File:        ReclsException.java
 *
 * Purpose:     Implementation of the org.recls.ReclsException class
 *
 * Created      21st November 2003
 * Updated:     16th June 2006
 *
 * Home:        http://recls.org/
 *
 * Copyright (c) 2003-2006, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


package org.recls;

import java.lang.Exception;

/** This class is thrown to represent recls-specific exceptional conditions
 */
public class ReclsException
    extends Exception
{
    /** Constructs an exception from the given message and error code
     */
    public ReclsException(String message, int code)
    {
        super(message);

        m_code = code;
    }

    /** Returns the value of the error code associated with the exception
     */
    public int getCode()
    {
        return m_code;
    }

    /** Translates a given error code to a human-readable string
     */
    public static native String translateCode(int code);

    /** Translates the assocated error code to a human-readable string
     */
    public String translateCode()
    {
        return translateCode(m_code);
    }

/// Members
    private int     m_code;
}
