/* /////////////////////////////////////////////////////////////////////////////
 * File:        Search.java
 *
 * Purpose:     Implementation of the org.recls.Search class
 *
 * Created      21st November 2003
 * Updated:     16th June 2006
 *
 * Home:        http://recls.org/
 *
 * Copyright (c) 2003-2006, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


package org.recls;

import java.io.File;
import java.util.Enumeration;

/** This class represents a recursive search
 */
public abstract class Search
    implements Enumeration
{
/// Constants
    public static final int RECLS_F_FILES						=   0x00000001; /*!< Include files in search. Included by default if none specified */
    public static final int RECLS_F_DIRECTORIES					=   0x00000002; /*!< Include directories in search. Not currently supported. */
    public static final int RECLS_F_LINKS						=   0x00000004; /*!< Include links in search. Ignored in Win32. */
    public static final int RECLS_F_DEVICES						=   0x00000008; /*!< Include devices in search. Not currently supported. */
//  public static final int RECLS_F_TYPEMASK					=   0x00000FFF;
    public static final int RECLS_F_RECURSIVE					=   0x00010000; /*!< Searches given directory and all sub-directories */
    public static final int RECLS_F_NO_FOLLOW_LINKS				=   0x00020000; /*!< Does not expand links */
    public static final int RECLS_F_DIRECTORY_PARTS				=   0x00040000; /*!< Fills out the directory parts. Supported from version 1.1.1 onwards. */
    public static final int RECLS_F_DETAILS_LATER				=   0x00080000; /*!< Does not fill out anything other than the path. Not currently supported. */
    public static final int RECLS_F_PASSIVE_FTP					=   0x00100000; /*!< Passive mode in FTP */
    public static final int RECLS_F_MARK_DIRS                   =   0x00200000; /*!< Marks the directories with a trailing slash. */
    public static final int RECLS_F_ALLOW_REPARSE_DIRS          =   0x00400000; /*!< Allow Win32 reparse point directories to be examined (which can cause infinite loops). */
    public static final int RECLS_F_USE_TILDE_ON_NO_SEARCHROOT  =   0x04000000; /*!< Interprets a NULL or empty searchRoot as the home directory, rather than the current directory. */

/// Class methods
	/** Returns an array of the roots for the current system.
	 *
	 * @deprecated Use FileSearch.getRoots() instead
	 */
//	public String[] getRoots()
//	{
//		return FileSearch.getRoots();
//	}

/// Construction
    public Search(int hSearch)
    {
        m_hSearch = hSearch;
    }

    /** Releases the resources owned by the search
     */
    public native void close();
    /** Releases the resources owned by the search
     */
    public void finalize()
    {
        close();
    }

/// Enumeration methods
    /** Tests if this enumeration contains more elements
     */
    public final boolean hasMoreElements()
    {
        return hasMoreElements_();
    }
        
    /** Returns the next element of this enumeration if this enumeration object has at least one more element to provide. 
     *
     * \note This throws java.lang.NoSuchElementException if hasMoreElements() would return false.
     */
    public final Object nextElement()
    {
        return nextElement_();
    }

/// Native method implementations
    private native boolean  hasMoreElements_();
    private native Object   nextElement_();

// Implementation
    static 
    {
        /** Causes the system to load the CDDisc (.dll, .so, etc) module */
        System.loadLibrary("recls.jni");
    }

/// Members
    private int	m_hSearch;  // The opaque handle to the entry
    private int m_rc;
}

/* ////////////////////////////////////////////////////////////////////////// */
