/* /////////////////////////////////////////////////////////////////////////////
 * File:        recls_wininet_dl.cpp
 *
 * Purpose:     Delay-loading functionality for the recls library.
 *
 * Created:     9th September 2004
 * Updated:     5th October 2006
 *
 * Home:        http://recls.org/
 *
 * Copyright (c) 2004-2006, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the name(s) of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/* File version */
#ifndef RECLS_DOCUMENTATION_SKIP_SECTION
# define RECLS_VER_CPP_RECLS_WININET_DL_MAJOR       2
# define RECLS_VER_CPP_RECLS_WININET_DL_MINOR       0
# define RECLS_VER_CPP_RECLS_WININET_DL_REVISION    1
# define RECLS_VER_CPP_RECLS_WININET_DL_EDIT        11
#endif /* !RECLS_DOCUMENTATION_SKIP_SECTION */

/** \file recls_wininet_dl.cpp Delay-loading functionality for the recls library */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <recls/recls.h>
#include "recls_wininet_dl.h"
#include "recls_impl.h"

#include "recls_debug.h"

#include <stdlib.h>

/* ////////////////////////////////////////////////////////////////////////// */

#if defined(RECLS_DELAY_LOAD_WININET)

/* /////////////////////////////////////////////////////////////////////////////
 * Macros, definitions and typedefs
 */

#undef FtpGetCurrentDirectoryA
#undef FtpGetCurrentDirectoryW
#undef InternetOpenA
#undef InternetOpenW
#undef InternetConnectA
#undef InternetConnectW
#undef InternetFindNextFileA
#undef InternetFindNextFileW
#undef FtpFindFirstFileA
#undef FtpFindFirstFileW
#undef InternetCloseHandle

#define HINTERNET               LPVOID
#define INTERNET_PORT           WORD

#ifndef INTERNETAPI_
# define INTERNETAPI_(type)     EXTERN_C type STDAPICALLTYPE
#endif /* !INTERNETAPI_ */

#ifndef BOOLAPI
# define BOOLAPI                INTERNETAPI_(BOOL)
#endif /* !BOOLAPI */

#ifndef HINTERNETAPI
# define HINTERNETAPI           INTERNETAPI_(HINTERNET)
#endif /* !HINTERNETAPI */

/* /////////////////////////////////////////////////////////////////////////////
 * Helper Functions
 */

static HINSTANCE    s_hinstWinInet  =   NULL;

static HINSTANCE    load_or_quit()
{
    HINSTANCE   hinstWinInet    =   ::LoadLibrary("WinInet");

    if(NULL == hinstWinInet)
    {
        ::OutputDebugStringA("Failed to load WININET.DLL!\n");

        ::ExitProcess(EXIT_FAILURE);
    }

    return hinstWinInet;
}

static HINSTANCE get_WinInet()
{
    static HINSTANCE    s_hinstWinInet  =   load_or_quit();

    return s_hinstWinInet;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Delay-load "API" Functions
 */

/* ////////////////////////////////////////////////////////////////////////// */

#define recls_dl_FN_1(R, F, A0, RV)                                             \
                                                                                \
    union U_##F                                                                 \
    {                                                                           \
        R (STDAPICALLTYPE   *pfn)(A0);                                          \
        FARPROC             fp;                                                 \
                                                                                \
        U_##F(HINSTANCE hinst)                                                  \
        {                                                                       \
            this->fp = (NULL != hinst) ? ::GetProcAddress(hinst, #F) : NULL;    \
        }                                                                       \
    };                                                                          \
    extern "C" R STDAPICALLTYPE recls_dl_ ## F(A0 a0)                           \
    {                                                                           \
        static  U_##F  u(get_WinInet());                                        \
                                                                                \
        return (NULL != u.fp) ? (*u.pfn)(a0) : RV;                              \
    }

#define recls_dl_FN_2(R, F, A0, A1, RV)                                         \
                                                                                \
    union U_##F                                                                 \
    {                                                                           \
        R (STDAPICALLTYPE   *pfn)(A0, A1);                                      \
        FARPROC             fp;                                                 \
                                                                                \
        U_##F(HINSTANCE hinst)                                                  \
        {                                                                       \
            this->fp = (NULL != hinst) ? ::GetProcAddress(hinst, #F) : NULL;    \
        }                                                                       \
    };                                                                          \
    extern "C" R STDAPICALLTYPE recls_dl_ ## F(A0 a0, A1 a1)                    \
    {                                                                           \
        static  U_##F  u(get_WinInet());                                        \
                                                                                \
        return (NULL != u.fp) ? (*u.pfn)(a0, a1) : RV;                          \
    }

#define recls_dl_FN_3(R, F, A0, A1, A2, RV)                                     \
                                                                                \
    union U_##F                                                                 \
    {                                                                           \
        R (STDAPICALLTYPE   *pfn)(A0, A1, A2);                                  \
        FARPROC             fp;                                                 \
                                                                                \
        U_##F(HINSTANCE hinst)                                                  \
        {                                                                       \
            this->fp = (NULL != hinst) ? ::GetProcAddress(hinst, #F) : NULL;    \
        }                                                                       \
    };                                                                          \
    extern "C" R STDAPICALLTYPE recls_dl_ ## F(A0 a0, A1 a1, A2 a2)             \
    {                                                                           \
        static  U_##F  u(get_WinInet());                                        \
                                                                                \
        return (NULL != u.fp) ? (*u.pfn)(a0, a1, a2) : RV;                      \
    }

#define recls_dl_FN_5(R, F, A0, A1, A2, A3, A4, RV)                                 \
                                                                                    \
    union U_##F                                                                     \
    {                                                                               \
        R (STDAPICALLTYPE   *pfn)(A0, A1, A2, A3, A4);                              \
        FARPROC             fp;                                                     \
                                                                                    \
        U_##F(HINSTANCE hinst)                                                      \
        {                                                                           \
            this->fp = (NULL != hinst) ? ::GetProcAddress(hinst, #F) : NULL;        \
        }                                                                           \
    };                                                                              \
    extern "C" R STDAPICALLTYPE recls_dl_ ## F(A0 a0, A1 a1, A2 a2, A3 a3, A4 a4)   \
    {                                                                               \
        static  U_##F  u(get_WinInet());                                            \
                                                                                    \
        return (NULL != u.fp) ? (*u.pfn)(a0, a1, a2, a3, a4) : RV;                  \
    }

#define recls_dl_FN_8(R, F, A0, A1, A2, A3, A4, A5, A6, A7, RV)                                         \
                                                                                                        \
    union U_##F                                                                                         \
    {                                                                                                   \
        R (STDAPICALLTYPE   *pfn)(A0, A1, A2, A3, A4, A5, A6, A7);                                      \
        FARPROC             fp;                                                                         \
                                                                                                        \
        U_##F(HINSTANCE hinst)                                                                          \
        {                                                                                               \
            this->fp = (NULL != hinst) ? ::GetProcAddress(hinst, #F) : NULL;                            \
        }                                                                                               \
    };                                                                                                  \
    extern "C" R STDAPICALLTYPE recls_dl_ ## F(A0 a0, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7)  \
    {                                                                                                   \
        static  U_##F  u(get_WinInet());                                                                \
                                                                                                        \
        return (NULL != u.fp) ? (*u.pfn)(a0, a1, a2, a3, a4, a5, a6, a7) : RV;                          \
    }

/* ////////////////////////////////////////////////////////////////////////// */

recls_dl_FN_1(BOOL, InternetCloseHandle, HINTERNET, FALSE);

recls_dl_FN_2(BOOL, InternetFindNextFileA, HINTERNET, LPVOID, FALSE);
recls_dl_FN_2(BOOL, InternetFindNextFileW, HINTERNET, LPVOID, FALSE);

recls_dl_FN_5(HINTERNET, FtpFindFirstFileA, HINTERNET, LPCSTR, LPWIN32_FIND_DATAA, DWORD, DWORD_PTR, NULL)
recls_dl_FN_5(HINTERNET, FtpFindFirstFileW, HINTERNET, LPCWSTR, LPWIN32_FIND_DATAW, DWORD, DWORD_PTR, NULL)

recls_dl_FN_3(BOOL, FtpGetCurrentDirectoryA, HINTERNET, LPSTR, LPDWORD, FALSE);
recls_dl_FN_3(BOOL, FtpGetCurrentDirectoryW, HINTERNET, LPWSTR, LPDWORD, FALSE);

recls_dl_FN_5(HINTERNET, InternetOpenA, LPCSTR, DWORD, LPCSTR, LPCSTR, DWORD, NULL)
recls_dl_FN_5(HINTERNET, InternetOpenW, LPCWSTR, DWORD, LPCWSTR, LPCWSTR, DWORD, NULL)

recls_dl_FN_8(HINTERNET, InternetConnectA, HINTERNET, LPCSTR, INTERNET_PORT, LPCSTR, LPCSTR, DWORD, DWORD, DWORD_PTR, NULL)
recls_dl_FN_8(HINTERNET, InternetConnectW, HINTERNET, LPCWSTR, INTERNET_PORT, LPCWSTR, LPCWSTR, DWORD, DWORD, DWORD_PTR, NULL)

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* RECLS_DELAY_LOAD_WININET */

/* ////////////////////////////////////////////////////////////////////////// */
