/* /////////////////////////////////////////////////////////////////////////////
 * File:        COM.cpp
 *
 * Purpose:     Implementation file for the COM project.
 *
 * Created:     19th December 2003
 * Updated:     22nd April 2007
 *
 * Status:      Wizard-generated
 *
 * License:     (Licensed under the Synesis Software Open License)
 *
 *              Copyright (c) 2003-2007, Synesis Software Pty Ltd.
 *              All rights reserved.
 *
 *              www:        http://www.synesis.com.au/software
 *
 *              This source code is placed into the public domain 2003
 *              by Synesis Software Pty Ltd. There are no restrictions
 *              whatsoever to your use of the software. 
 *
 *              This source code is provided by Synesis Software Pty Ltd "as is"
 *              and any warranties, whether expressed or implied, including, but
 *              not limited to, the implied warranties of merchantability and
 *              fitness for a particular purpose are disclaimed. In no event
 *              shall the Synesis Software Pty Ltd be liable for any direct,
 *              indirect, incidental, special, exemplary, or consequential
 *              damages (including, but not limited to, procurement of
 *              substitute goods or services; loss of use, data, or profits; or
 *              business interruption) however caused and on any theory of
 *              liability, whether in contract, strict liability, or tort
 *              (including negligence or otherwise) arising in any way out of
 *              the use of this software, even if advised of the possibility of
 *              such damage. 
 *
 *              Neither the name of Synesis Software Pty Ltd nor the names of
 *              any subdivisions, employees or agents of Synesis Software Pty
 *              Ltd, nor the names of any other contributors to this software
 *              may be used to endorse or promote products derived from this
 *              software without specific prior written permission. 
 *
 * ////////////////////////////////////////////////////////////////////////// */

/** \file COM.cpp Test program for the \ref group_recls_com "recls COM mapping" */

#include <stlsoft/stlsoft.h>    /* If the compiler cannot find this, you are not using STLSoft 1.9.1 or later, as required. */
#if !defined(_STLSOFT_VER_1_9_1) || \
   _STLSOFT_VER < _STLSOFT_VER_1_9_1
# error Requires STLSoft 1.9.1, or later. (www.stlsoft.org/downloads.html)
#endif /* STLSoft version */

#include <atlstl/atlstl.hpp>

#include <comstl/comstl.h>
#include <comstl/automation/ole_object.hpp>
#include <comstl/collections/enumeration_policies.hpp>
#include <comstl/collections/enumerator_sequence.hpp>
#include <comstl/collections/collection_sequence.hpp>
#include <comstl/conversion/method_cast.hpp>
#include <comstl/interface/interface_cast.hpp>
#include <comstl/util/creation_functions.hpp>
#include <comstl/util/initialisers.hpp>
#include <comstl/util/value_policies.hpp>

#include <winstl/winstl.h>
#include <winstl/conversion/char_conversions.hpp>
#include <winstl/system/console_functions.h>

#include <recls.com.h>
#include <recls.com_i.c>

/* Standard C++ Header Files */
# include <iostream>
# include <iomanip>

using std::cout;
using std::wcout;
using std::cerr;
using std::endl;

/* Standard C Header Files */
#include <stdio.h>
#include <stdlib.h>

#ifdef _MSC_VER
# include <CrtDbg.h>
#endif /* _MSC_VER */

//#define USE_ENUMVARIANT
#define USE_ENUMENTRY

/* /////////////////////////////////////////////////////////////////////////////
 * Helper functions
 */

template<typename S>
S &operator <<(S &s, CComBSTR const &bstr)
{
    wchar_t const   *str = bstr;

    return s << str;
}

namespace comstl
{
    inline CComBSTR &transfer_resource(CComBSTR &r, BSTR v)
    {
        comstl_message_assert("Resource destination not empty!", NULL == r.m_str);
        r.m_str = v;

        return r;
    }
}

/* /////////////////////////////////////////////////////////////////////////////
 * Progress class
 */

interface FileSearchProgress
    : public IFileSearchProgress
{
/// \name Construction
/// @{
public:
    FileSearchProgress()
        : m_extent(0)
        , m_refs(0)
    {}
    ~FileSearchProgress()
    {
        STLSOFT_ASSERT(0 == m_refs);
    }
/// @}

/// \name Operations
/// @{
public:
    void Reset()
    {
        const size_t    consoleWidth    =   winstl::get_console_width();
        size_t          n               =   m_extent;

        if(n > consoleWidth - 1)
        {
            n = consoleWidth - 1;
        }

        stlsoft::auto_buffer<char>  blanks(1 + n);
        stlsoft::auto_buffer<char>  spaces(1 + n);

        memset(&blanks[0], '\b', blanks.size() - 1);
        memset(&spaces[0], ' ', spaces.size() - 1);

        blanks[blanks.size() - 1] = '\0';
        spaces[spaces.size() - 1] = '\0';

        fprintf(stdout, "%.*s%.*s%.*s", (int)blanks.size(), &blanks[0], (int)spaces.size(), &spaces[0], (int)blanks.size(), &blanks[0]);

        m_extent = 0;
    }
/// @}

/// \name IUnknown methods
/// @{
private:
    STDMETHODIMP QueryInterface(REFIID riid, void **ppvObject)
    {
        if( IID_IUnknown == riid ||
            IID_IFileSearchProgress == riid)
        {
            *ppvObject = this;

            static_cast<LPUNKNOWN>(*ppvObject)->AddRef();

            return S_OK;
        }
        else
        {
            return E_NOINTERFACE;
        }
    }

    STDMETHODIMP_(ULONG)    AddRef()
    {
        return ++m_refs;
    }
    STDMETHODIMP_(ULONG)    Release()
    {
        COMSTL_ASSERT(0 < m_refs);

        return --m_refs;
    }
/// @}

/// \name IFileSearchProgress methods
/// @{
private:
    STDMETHODIMP OnDirectoryScan(/* [in] */ BSTR dirPath, /* [out, retval] */ VARIANT_BOOL *bCancel)
    {
        Reset();

        m_extent = ::SysStringLen(dirPath);

        const size_t    consoleWidth    =   winstl::get_console_width();
        size_t          n               =   m_extent;

        if(n > consoleWidth - 1)
        {
            n = consoleWidth - 1;
        }

        fwprintf(stdout, L"%.*s", (int)n, dirPath);

        *bCancel = VARIANT_FALSE;

        return S_OK;
    }
/// @}

/// \name Members
/// @{
private:
    size_t  m_extent;
    LONG    m_refs;
/// @}

/// \name Not to be implemented
/// @{
private:
    FileSearchProgress(FileSearchProgress const &);
    FileSearchProgress &operator =(FileSearchProgress const &);
/// @}
};

/* /////////////////////////////////////////////////////////////////////////////
 * Forward declarations
 */

static void usage(int bExit, char const *reason);
static int processRoot(IFileSearch *search, CComBSTR const &rootDir, CComBSTR const &pattern, long flags, int bSuccinct, IFileSearchProgress *fsp);
static int processRoot(IFtpSearch *search, CComBSTR const &host, CComBSTR const &username, CComBSTR const &password, CComBSTR const &rootDir, CComBSTR const &pattern, long flags, int bSuccinct, int bFTP);
static int processCollection(ISearchCollection *psc, long flags, int bSuccinct, int bFTP);
static void run_unittests();

/* /////////////////////////////////////////////////////////////////////////////
 * Main
 */

FileSearchProgress  s_fsp;

/* /////////////////////////////////////////////////////////////////////////////
 * Main
 */

/** Program main */
int main_(int argc, char *argv[])
{
    run_unittests();

    int             iRet            =   0;
    int             i;
    int             totalFound      =   0;
    int             bAllHardDrives  =   false;
    char const      *host           =   NULL;
    char const      *username       =   NULL;
    char const      *password       =   NULL;
    char const      *pattern        =   NULL;
    char const      *rootDir        =   NULL;
    long            flags           =   RECLS_F_RECURSIVE;
    int             bSuccinct       =   false;

    for(i = 1; i < argc; ++i)
    {
        const char  *arg    =   argv[i];

        if(arg[0] == '-')
        {
            if(arg[1] == '-')
            {
                /* -- arguments */
                if(0 == ::strcmp(arg, "--home"))
                {
                    rootDir =   "";
                    flags   |=  RECLS_F_USE_TILDE_ON_NO_SEARCHROOT;
                }
            }
            else
            {
                /* - arguments */
                switch(arg[1])
                {
                    case    '?':
                        usage(1, NULL);
                        break;
                    case    'R':    /* Do not recurse */
                        flags &= ~(RECLS_F_RECURSIVE);
                        break;
                    case    'p':    /* Show directory parts */
                        flags |= RECLS_F_DIRECTORY_PARTS;
                        break;
                    case    'f':    /* Find files */
                        flags |= RECLS_F_FILES;
                        break;
                    case    'd':    /* Find directories */
                        flags |= RECLS_F_DIRECTORIES;
                        break;
                    case    'h':    /* Searches from hard drives */
                        bAllHardDrives = true;
                        break;
                    case    'm':    /* Mark directories with a trailing slash */
                        flags |= RECLS_F_MARK_DIRS;
                        break;
                    case    'r':    /* Allow reparse points */
                        flags |= RECLS_F_ALLOW_REPARSE_DIRS;
                        break;
                    case    's':    /* Show only the full path; WHEREIS functionality */
                        bSuccinct = true;
                        break;
                    case    'H':    /* FTP host */
                        host = arg + 2;
                        break;
                    case    'U':    /* FTP username */
                        username = arg + 2;
                        break;
                    case    'P':    /* FTP password */
                        password = arg + 2;
                        break;
                    default:
                        usage(1, "Invalid argument(s) specified");
                        break;
                }
            }
        }
        else
        {
            /* other arguments */
            if(NULL == pattern)
            {
                pattern = arg;
            }
            else if(NULL == rootDir)
            {
                rootDir = arg;
            }
            else
            {
                usage(1, "Invalid argument(s) specified");
            }
        }
    }

    if( NULL != password &&
        (   NULL == host ||
            NULL == username))
    {
        usage(1, "Must specify host and username if specifying password");
    }

    if( NULL != username &&
        NULL == host)
    {
        usage(1, "Must specify host if specifying username");
    }

    if( NULL != host &&
        bAllHardDrives)
    {
        usage(1, "-h flag meaningless to FTP searches");
    }

    if( NULL != host &&
        NULL == rootDir)
    {
        usage(1, "Must specify a root directory for FTP searches");
    }

    // Search for files if neither files or directories specified
    //
    // Even though this is not necessary, because the recls API provides the
    // same interpretation, it's best to be explicit.
    if(0 == (flags & (RECLS_F_FILES | RECLS_F_DIRECTORIES)))
    {
        flags |= RECLS_F_FILES;
    }

#if 0
    if(NULL == pattern)
    {
        pattern = Recls_GetWildcardsAll();
    }
#endif /* 0 */

    if(NULL == rootDir)
    {
        rootDir = ".";
    }

    if(NULL != host)
    {
        IFtpSearch             *search;
        HRESULT                 hr      =   ::CoCreateInstance(CLSID_FileSearch, NULL, CLSCTX_ALL, IID_IFtpSearch, (void**)&search);

        if(FAILED(hr))
        {
            cerr << "Failed to load the recls COM server! Please check your installation" << endl;
        }
        else
        {
//            CComBSTR    bstrPattern = comstl::method_cast<CComBSTR>(search, &IFileSearch::get_WildcardsAll);

            // The ATL conversion is not to be trusted for early versions, so we do it here.
            winstl::a2w host_(host);
            winstl::a2w username_(username);
            winstl::a2w password_(password);
            winstl::a2w rootDir_(rootDir);
            winstl::a2w pattern_(pattern);

            CComBSTR    bstrHost(stlsoft::c_str_ptr(host));
            CComBSTR    bstrUsername(stlsoft::c_str_ptr(username));
            CComBSTR    bstrPassword(stlsoft::c_str_ptr(password));
            CComBSTR    bstrRootDir(stlsoft::c_str_ptr(rootDir));
            CComBSTR    bstrPattern(stlsoft::c_str_ptr(pattern));

            if(0 == stlsoft::c_str_len(bstrPattern))
            {
                bstrPattern = L"*";
            }

            totalFound = processRoot(search, bstrHost, bstrUsername, bstrPassword, bstrRootDir, bstrPattern, flags, bSuccinct, true);

            search->Release();
        }
    }
    else
    {
        IFileSearch             *search;
        HRESULT                 hr      =   ::CoCreateInstance(CLSID_FileSearch, NULL, CLSCTX_ALL, IID_IFileSearch, (void**)&search);

        if(FAILED(hr))
        {
            cerr << "Failed to load the recls COM server! Please check your installation" << endl;
        }
        else
        {
            winstl::a2w pattern_(pattern);
            CComBSTR    bstrPattern(stlsoft::c_str_ptr(pattern));

            if(0 == stlsoft::c_str_len(bstrPattern))
            {
                bstrPattern = comstl::method_cast<CComBSTR>(search, &IFileSearch::get_WildcardsAll);;
            }

            if(bAllHardDrives)
            {
                LPUNKNOWN   punkRoots;

                hr = search->get_Roots(&punkRoots);

                if(SUCCEEDED(hr))
                {
                    typedef comstl::enumerator_sequence<IEnumString, LPOLESTR, comstl::LPOLESTR_policy> enum_sequence_t;

#if 0
                    enum_sequence_t             roots(comstl::interface_cast_addref<LPENUMSTRING>(punkRoots), false);
#else /* ? 0 */
                    hr = comstl::new_enum_by_dispid_policy<IUnknown>::acquire(punkRoots, &punkRoots);

                    enum_sequence_t             roots(comstl::interface_cast_addref<LPENUMSTRING>(punkRoots), false);
#endif /* 0 */
                    enum_sequence_t::iterator   begin   =   roots.begin();
                    enum_sequence_t::iterator   end     =   roots.end();

                    for(; begin != end; ++begin)
                    {
                        totalFound += processRoot(search, *begin, bstrPattern, flags, bSuccinct, &s_fsp);
                    }

                    punkRoots->Release();
                }
            }
            else
            {
                winstl::a2w rootDir_(rootDir);
                CComBSTR    bstrRootDir(stlsoft::c_str_ptr(rootDir));

                totalFound = processRoot(search, bstrRootDir, bstrPattern, flags, bSuccinct, &s_fsp);
            }

            search->Release();
        }
    }

    fprintf(stdout, "Total found: %d\n", totalFound);

    return iRet;
}

int main(int argc, char *argv[])
{
    int             iRet;

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemState    memState;
#endif /* _MSC_VER && _MSC_VER */

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemCheckpoint(&memState);
#endif /* _MSC_VER && _MSC_VER */

#if 0
    { for(size_t i = 0; i < 0xffffffff; ++i){} }
#endif /* 0 */

    try
    {
#if defined(_DEBUG) || \
    defined(__SYNSOFT_DBS_DEBUG)
        puts("COM: " STLSOFT_COMPILER_LABEL_STRING);
#endif /* debug */

        comstl::com_initialiser coinit;

        iRet = main_(argc, argv);
    }
    catch(std::exception &x)
    {
        fprintf(stderr, "Unhandled error: %s\n", x.what());

        iRet = EXIT_FAILURE;
    }
    catch(...)
    {
        fprintf(stderr, "Unhandled unknown error\n");

        iRet = EXIT_FAILURE;
    }

#if defined(_MSC_VER) && \
    defined(_DEBUG)
    _CrtMemDumpAllObjectsSince(&memState);
#endif /* _MSC_VER) && _DEBUG */

    return iRet;
}

/* /////////////////////////////////////////////////////////////////////////////
 * Function implementations
 */

/** Displays program usage */
void usage(int bExit, char const *reason)
{
    cerr << "recls COM (C++) Test Program: COM" << endl << endl;

    if(NULL != reason)
    {
        cerr << "  Error: " << reason << endl << endl;
    }

    cerr << "Usage: COM [-d] [-f] [-h] [-p] [-R] [-s] [-H<host>] [-U<username>] [-P<password>] [<pattern>] [<root-dir>]" << endl;
    cerr << "\t-R               -   does not recurse; recursive search is the default" << endl;
    cerr << "\t-p               -   evaluate and display directory parts" << endl;
    cerr << "\t-d               -   search for directories" << endl;
    cerr << "\t-f               -   search for files (this is the default, if -d is not specified" << endl;
    cerr << "\t-h               -   searches from hard drive roots. (Ignore <root-dir>.)" << endl;
    cerr << "\t-s               -   succinct; shows only the path" << endl;
    cerr << "\t-H<host>         -   performs an FTP search on the given host" << endl;
    cerr << "\t-U<username>     -   username to use to log on to <host>" << endl;
    cerr << "\t-P<password>     -   password to use to log on to <host>" << endl;
    cerr << "\t<pattern>        -   search pattern, e.g. \"*.cpp\"; default is to search for all files" << endl;
    cerr << "\t<root-dir>       -   root directory of search; default is current working directory" << endl;

    if(bExit)
    {
        exit(EXIT_FAILURE);
    }
}

int processRoot(IFtpSearch *search, CComBSTR const &host, CComBSTR const &username, CComBSTR const &password, CComBSTR const &rootDir, CComBSTR const &pattern, long flags, int bSuccinct, int bFTP)
{
    int         totalFound  =   0;
    IUnknown    *punkResults;
    HRESULT     hr          =   search->Search(host, username, password, rootDir, pattern, flags, &punkResults);

    if(SUCCEEDED(hr))
    {
        ISearchCollection   *psc = comstl::interface_cast_addref<ISearchCollection*>(punkResults);

        if(NULL != psc)
        {
            totalFound = processCollection(psc, flags, bSuccinct, bFTP);

            psc->Release();
        }

        punkResults->Release();
    }

    return totalFound;
}

int processRoot(IFileSearch *fs, CComBSTR const &rootDir, CComBSTR const &pattern, long flags, int bSuccinct, IFileSearchProgress *fsp)
{
    int             totalFound  =   0;
    IUnknown        *punkResults;
    IFileSearch2    *fs2;
    HRESULT         hr  =   fs->QueryInterface(IID_IFileSearch2, reinterpret_cast<void**>(&fs2));

    if(SUCCEEDED(hr))
    {
        hr = fs2->SearchProgress(rootDir, pattern, flags, fsp, &punkResults);

        fs2->Release();
    }
    else
    {
        hr = fs->Search(rootDir, pattern, flags, &punkResults);
    }

    if(SUCCEEDED(hr))
    {
        ISearchCollection   *psc = comstl::interface_cast_addref<ISearchCollection*>(punkResults);

        if(NULL != psc)
        {
            totalFound = processCollection(psc, flags, bSuccinct, false);

            psc->Release();
        }

        punkResults->Release();
    }

    return totalFound;
}

int processCollection(ISearchCollection *psc, long flags, int bSuccinct, int bFTP)
{
    int             totalFound      =   0;

#if defined(USE_ENUMVARIANT)
    typedef comstl::collection_sequence<    ISearchCollection
                                        ,   IEnumVARIANT
                                        ,   VARIANT
                                        ,   comstl::VARIANT_policy
                                        >           coll_seq_t;
#elif defined(USE_ENUMENTRY)
    typedef comstl::collection_sequence<    ISearchCollection
                                        ,   IEnumFileEntry
                                        ,   IFileEntry*
                                        ,   comstl::interface_policy<IFileEntry>
                                        >           coll_seq_t;
#else
# error Must specify USE_ENUMVARIANT or USE_ENUMENTRY
#endif /* 0 */

    coll_seq_t              collection(psc, true, /* bFTP ? 1 : 0 */1);
    coll_seq_t::iterator    b   =   collection.begin();
    coll_seq_t::iterator    e   =   collection.end();

    {
        coll_seq_t::iterator    b2   =   b;

        bool        isEqual     =   (b == b2);
        bool        isNotEqual  =   (b != b2);
        if(b2 != e)
        {
            IFileEntry  *pfe        =   *b2;
            CComBSTR    path        =   ::comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_Path);

            s_fsp.Reset();
            wcout << L"  " << path << endl;
        }

        COMSTL_ASSERT(isEqual != isNotEqual);
    }

    {
        coll_seq_t::iterator    b2   =   collection.begin();

        bool        isEqual     =   (b == b2);
        bool        isNotEqual  =   (b != b2);
        if(b2 != e)
        {
            IFileEntry  *pfe        =   *b2;
            CComBSTR    path        =   ::comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_Path);

            s_fsp.Reset();
            wcout << L"  " << path << endl;
        }

        COMSTL_ASSERT(isEqual != isNotEqual);
    }

//  exit(0);

    for(; b != e; ++b)
    {
        ++totalFound;

        s_fsp.Reset();

#if defined(USE_ENUMVARIANT)
        IFileEntry  *pfe = comstl::interface_cast_addref<IFileEntry*>((*b).punkVal);
#elif defined(USE_ENUMENTRY)
        IFileEntry  *pfe = *b;
#else
# error Must specify USE_ENUMVARIANT or USE_ENUMENTRY
#endif /* 0 */

        CComBSTR    path                    =   ::comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_Path);
        OLECHAR     drive                   =   ::comstl::method_cast<OLECHAR>(pfe, &IFileEntry::get_Drive);
        CComBSTR    directory               =   ::comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_Directory);
        CComBSTR    directoryPath           =   ::comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_DirectoryPath);
        CComBSTR    searchRelativePath;
        CComBSTR    uncDrive;
        LPUNKNOWN   directoryParts          =   ::comstl::method_cast<LPUNKNOWN>(pfe, &IFileEntry::get_DirectoryParts);
        CComBSTR    file                    =   ::comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_File);
        CComBSTR    shortFile               =   ::comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_ShortFile);
        CComBSTR    fileName                =   ::comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_FileName);
        CComBSTR    fileExt                 =   ::comstl::method_cast<CComBSTR>(pfe, &IFileEntry::get_FileExt);
        DATE        creationTime            =   ::comstl::method_cast<DATE>(pfe, &IFileEntry::get_CreationTime);
        DATE        modificationTime        =   ::comstl::method_cast<DATE>(pfe, &IFileEntry::get_ModificationTime);
        DATE        lastAccessTime          =   ::comstl::method_cast<DATE>(pfe, &IFileEntry::get_LastAccessTime);
        DATE        lastStatusChangeTime    =   ::comstl::method_cast<DATE>(pfe, &IFileEntry::get_LastStatusChangeTime);
        long        size                    =   ::comstl::method_cast<long>(pfe, &IFileEntry::get_Size);
        BOOL        isReadOnly              =   ::comstl::method_cast<BOOL>(pfe, &IFileEntry::get_IsReadOnly);
        BOOL        isDirectory             =   ::comstl::method_cast<BOOL>(pfe, &IFileEntry::get_IsDirectory);
        BOOL        isLink                  =   FALSE;
        BOOL        isUNC                   =   FALSE;

        if(::comstl::interface_cast_test<IFileEntry4*>(pfe))
        {
            IFileEntry4 *pfe4   =   ::comstl::interface_cast_addref<IFileEntry4*>(pfe);

            if(NULL != pfe4)
            {
                IFileEntry2 *pfe2   =   pfe4;

                uncDrive            =   ::comstl::method_cast<CComBSTR>(pfe2, &IFileEntry2::get_UNCDrive);
                isLink              =   ::comstl::method_cast<BOOL>(pfe2, &IFileEntry2::get_IsLink);
                isUNC               =   ::comstl::method_cast<BOOL>(pfe2, &IFileEntry2::get_IsUNC);
                searchRelativePath  =   ::comstl::method_cast<CComBSTR>(pfe4, &IFileEntry4::get_SearchRelativePath);

                pfe4->Release();
            }
        }

        wcout << L"  " << path << endl;

        if(!bSuccinct)
        {
            int off = 0;

            if(searchRelativePath.Length())
            {
                wcout << L"  " << std::setw(path.Length()) << searchRelativePath << endl;
            }

            wcout << L"  " << directoryPath << endl;
            if(isUNC)
            {
                wcout << L"  " << uncDrive << endl;
                off += uncDrive.Length();
            }
            else
            {
                wcout << L"  " << (wchar_t const *)winstl::a2w((char const*)&drive, 1) << endl;
                off += 2;
            }
            wcout << L"  " << std::setw(off + directory.Length()) << directory << std::setw(1) << endl;

            if( (flags & RECLS_F_DIRECTORY_PARTS) == RECLS_F_DIRECTORY_PARTS &&
                NULL != directoryParts)
            {
                typedef comstl::collection_sequence<    IDirectoryPartsCollection
                                                    ,   IEnumVARIANT
                                                    ,   VARIANT
                                                    ,   comstl::VARIANT_policy
                                                    ,   VARIANT const &
                                                    ,   comstl::forward_cloning_policy<IEnumVARIANT>
                                                    ,   2
                                                    >           parts_seq_t;

                parts_seq_t             dirParts(comstl::interface_cast_addref<IDirectoryPartsCollection*>(directoryParts), false);
                parts_seq_t::iterator   b   =   dirParts.begin();
                parts_seq_t::iterator   e   =   dirParts.end();

                for(; b != e; ++b)
                {
                    CComVariant vPart(*b);  vPart.ChangeType(VT_BSTR);
                    CComBSTR    dirPart(vPart.bstrVal);

                    off += dirPart.Length();
                    wcout << L"  " << std::setw(off) << dirPart << std::setw(1) << endl;
                }

                directoryParts->Release();
            }
            else
            {
                off += directory.Length();
            }

            off += file.Length();
            wcout << L"  " << std::setw(off) << file << std::setw(1) << endl;
            if(fileExt.Length() > 0)
            {
                wcout << L"  " << std::setw(off - (1 + fileExt.Length())) << fileName << std::setw(1) << endl;
                wcout << L"  " << std::setw(off) << fileExt << std::setw(1) << endl;
            }
            else
            {
                wcout << L"  " << std::setw(off) << fileName << std::setw(1) << endl;
            }

            if(isReadOnly)
            {
                wcout << L"    - Read-only" << endl;
            }
            if(isDirectory)
            {
                wcout << L"    - Directory" << endl;
            }
            else
            {
                wcout << L"    - File" << endl;
            }
            if(isLink)
            {
                wcout << L"    - Link" << endl;
            }
            if(isUNC)
            {
                wcout << L"    - UNC" << endl;
            }
        }

#if defined(USE_ENUMVARIANT)
        pfe->Release();
#endif /* USE_ENUMVARIANT */
    }

    return totalFound;
}

/* /////////////////////////////////////////////////////////////////////////
 * Unit-tests
 */

bool check_combine(LPCWSTR requiredResult, LPCWSTR path1, LPCWSTR path2)
{
    IFileSearch3    *search;
    HRESULT         hr = comstl::co_create_instance(CLSID_FileSearch, &search);

    if(SUCCEEDED(hr))
    {
        BSTR    actualResult;

        hr = search->CombinePaths((BSTR)path1, (BSTR)path2, &actualResult);

        if(SUCCEEDED(hr))
        {
            if(0 != ::wcscmp(requiredResult, actualResult))
            {
                hr = E_FAIL;
            }

            ::SysFreeString(actualResult);
        }

        search->Release();
    }

    return SUCCEEDED(hr);
}

static void run_unittests()
{
    // 1. CombinePaths

    COMSTL_ASSERT(check_combine(L"abc\\def", L"abc", L"def"));
    COMSTL_ASSERT(check_combine(L"abc\\def", L"abc\\", L"def"));
    COMSTL_ASSERT(check_combine(L"a\\b\\c\\def", L"a\\b\\c", L"def"));
    COMSTL_ASSERT(check_combine(L"abc", L"abc", NULL));
    COMSTL_ASSERT(check_combine(L"def", NULL, L"def"));
    COMSTL_ASSERT(check_combine(L"\\def", L"abc", L"\\def"));
    COMSTL_ASSERT(check_combine(L"H:\\abc\\def", L"H:\\abc", L"def"));
    COMSTL_ASSERT(check_combine(L"H:\\abc\\def", L"H:\\abc\\", L"def"));
    COMSTL_ASSERT(check_combine(L"H:\\abc\\def\\", L"H:\\abc", L"def\\"));
    COMSTL_ASSERT(check_combine(L"H:\\abc\\def\\", L"H:\\abc\\", L"def\\"));

    COMSTL_ASSERT(check_combine(L"abc\\def", L"abc", L"def"));
    COMSTL_ASSERT(check_combine(L"abc/def", L"abc/", L"def"));
    COMSTL_ASSERT(check_combine(L"a/b/c/def", L"a/b/c", L"def"));
    COMSTL_ASSERT(check_combine(L"abc", L"abc", NULL));
    COMSTL_ASSERT(check_combine(L"def", NULL, L"def"));
    COMSTL_ASSERT(check_combine(L"/def", L"abc", L"/def"));
    COMSTL_ASSERT(check_combine(L"H:/abc/def", L"H:/abc", L"def"));
    COMSTL_ASSERT(check_combine(L"H:/abc/def", L"H:/abc/", L"def"));
    COMSTL_ASSERT(check_combine(L"H:/abc/def/", L"H:/abc", L"def/"));
    COMSTL_ASSERT(check_combine(L"H:/abc/def/", L"H:/abc/", L"def/"));

#if 0
    // 2. SqueezePath

    COMSTL_ASSERT(L"" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 0));
    COMSTL_ASSERT(L"g" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 1));
    COMSTL_ASSERT(L"gh" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 2));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 3));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 4));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 5));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 6));
    COMSTL_ASSERT(L"...\\ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 7));
    COMSTL_ASSERT(L"a...\\ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 8));
    COMSTL_ASSERT(L"ab...\\ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 9));
    COMSTL_ASSERT(L"abc...\\ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 10));
    COMSTL_ASSERT(L"abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 11));
    COMSTL_ASSERT(L"abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 12));
    COMSTL_ASSERT(L"abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 13));
    COMSTL_ASSERT(L"abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("abc\\def\\ghi", 100));

    COMSTL_ASSERT(L"" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 0));
    COMSTL_ASSERT(L"g" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 1));
    COMSTL_ASSERT(L"gh" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 2));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 3));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 4));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 5));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 6));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 7));
    COMSTL_ASSERT(L"\\...\\ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 8));
    COMSTL_ASSERT(L"\\a...\\ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 9));
    COMSTL_ASSERT(L"\\ab...\\ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 10));
    COMSTL_ASSERT(L"\\abc...\\ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 11));
    COMSTL_ASSERT(L"\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 12));
    COMSTL_ASSERT(L"\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 13));
    COMSTL_ASSERT(L"\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 14));
    COMSTL_ASSERT(L"\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("\\abc\\def\\ghi", 100));

    COMSTL_ASSERT(L"" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 0));
    COMSTL_ASSERT(L"g" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 1));
    COMSTL_ASSERT(L"gh" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 2));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 3));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 4));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 5));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 6));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 7));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 8));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 9));
    COMSTL_ASSERT(L"C:\\...\\ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 10));
    COMSTL_ASSERT(L"C:\\a...\\ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 11));
    COMSTL_ASSERT(L"C:\\ab...\\ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 12));
    COMSTL_ASSERT(L"C:\\abc...\\ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 13));
    COMSTL_ASSERT(L"C:\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 14));
    COMSTL_ASSERT(L"C:\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 15));
    COMSTL_ASSERT(L"C:\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 16));
    COMSTL_ASSERT(L"C:\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("C:\\abc\\def\\ghi", 1100));

    COMSTL_ASSERT(L"" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 0));
    COMSTL_ASSERT(L"g" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 1));
    COMSTL_ASSERT(L"gh" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 2));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 3));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 4));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 5));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 6));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 7));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 8));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 9));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 10));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 11));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 12));
    COMSTL_ASSERT(L"ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 13));
    COMSTL_ASSERT(L"\\\\host\\...\\ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 14));
    COMSTL_ASSERT(L"\\\\host\\a...\\ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 15));
    COMSTL_ASSERT(L"\\\\host\\ab...\\ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 16));
    COMSTL_ASSERT(L"\\\\host\\abc...\\ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 17));
    COMSTL_ASSERT(L"\\\\host\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 18));
    COMSTL_ASSERT(L"\\\\host\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 19));
    COMSTL_ASSERT(L"\\\\host\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 20));
    COMSTL_ASSERT(L"\\\\host\\abc\\def\\ghi" == recls::cpp::FileSearch::SqueezePath("\\\\host\\abc\\def\\ghi", 1100));

    // 3. Relative path
    COMSTL_ASSERT(L"..\\ghi" == recls::cpp::FileSearch::RelativePath("/abc/def", "/abc/ghi"));
    COMSTL_ASSERT(L"..\\defghi" == recls::cpp::FileSearch::RelativePath("h:\\abc\\def", "H:\\abc\\defghi"));

    COMSTL_ASSERT(L"C:\\abc\\defghi" == recls::cpp::FileSearch::RelativePath("h:\\abc\\def", "C:\\abc\\defghi"));

    ::SetCurrentDirectoryA("C:\\");
    COMSTL_ASSERT(L"..\\defghi" == recls::cpp::FileSearch::RelativePath("abc\\def", "C:\\abc\\defghi"));
    COMSTL_ASSERT(L"H:\\abc\\defghi" == recls::cpp::FileSearch::RelativePath("", "H:\\abc\\defghi"));
    COMSTL_ASSERT(L"abc\\defghi" == recls::cpp::FileSearch::RelativePath("", "C:\\abc\\defghi"));

    COMSTL_ASSERT(L"def" == recls::cpp::FileSearch::RelativePath(".", "def"));

    COMSTL_ASSERT(L"..\\..\\lib\\recls" == recls::cpp::FileSearch::RelativePath("/usr/include/recls", "/usr/lib/recls"));
    
    COMSTL_ASSERT(L"..\\def" == recls::cpp::FileSearch::RelativePath("abc", "def"));
#endif /* 0 */
}

/* //////////////////////////////////////////////////////////////////////////
 * End of file
 */
